/*******************************************************************
 *                                                                 *
 * File: dcache.c                          Date: 3/11/95           *
 *                                                                 *
 * Disclaimer:                                                     *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY,   *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR       *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no     *
 *              guarantee or representations regarding the use     *
 *              of, or the results of the use of, the software     *
 *              and documentation in terms of correctness,         *
 *              accuracy, reliability, currentness, or             *
 *              otherwise,  and you rely on the software,          *
 *              documentation, and results solely at your own      *
 *              risk.                                              *
 *                                                                 * 
 * Overview:                                                       *
 *                                                                 *
 *  The purpose of this program is to show the performance         *
 *  advantage that can be obtained by the use of the data cache.   *
 *                                                                 *
 * Revision History:                                               *
 * ----------------------------------------------------------------*
 *******************************************************************/

#include <stdio.h>
#include "system.h"


/* function declaration */
void timed_routine();
void data_cache_off();
void data_cache_on();


/*******************************************************************
 * File:  dcache.c                         Author:  Intel Corp.    *
 *                                                                 *
 * Procedure:  Main                                                *
 *                                                                 *
 * Description:                                                    *
 *                                                                 *
 *      This program will execute a routine that will calculate    *
 * the prime numbers.  This routine will be timed with the data    *
 * cache off and then with the data cache on.  The time required   *
 * to perform this function will be displayed for both             *
 * circumstances.  This program contains the procedures bentime    *
 * and init_bentime which are contained within the GNU libraries.  *
 *                                                                 *
 *******************************************************************/
 main()
{
   unsigned int    overhead,    /* initial time */
                   start,       /* start time   */
                   stop,        /* end time     */
                   elapsed;     /* final time   */


   printf("\n\nTimed routine calculates prime numbers.\n\n");



/* Disables data cache  */
   data_cache_off ();

   printf("Elapsed time for routine with data cache off =\n ");


/* Begin timer count */
   overhead = init_bentime(0);   /* initializes timer, overhead provides *
                                  * the time to make the call            */
   start = bentime();            /* reads the initial timer value */

/* Begin timed routine */
   timed_routine();

/* End count */
   stop = bentime();            /* reads the ending timer value */
   
   elapsed = (stop - start)  ;  /* computes final time */
   printf("%11d microseconds\n\n",elapsed);


/* Enable the data cache */
   data_cache_on();  

   printf("Elapsed time for routine with data cache on = \n");

/* Initialize count */
   overhead = init_bentime(0);   /* initializes timer, overhead provides *
                                  * the time to make the call            */ 
   start = bentime();            /* reads the inital timer value */

  
/* Begin timed routine */
   timed_routine();

/* Stop count */
   stop = bentime();            /* reads the ending timer value */
   
   elapsed = (stop - start)  ;  /* computes final time */
   printf(" %11d microseconds\n\n",elapsed);

   

}  /* end main */


/***************************************************************
*  File:         dcache.c                Author: Intel Corp.   *
*                                                              *
*  Procedure:    data_cache_off                                *
*                                                              *
*  Description:                                                *
*                This procedure call the dcctl macro.  It      *
*                disables the data cache.                      *
*                                                              *
*    Inputs    - None                                          *
*    Outputs   - None                                          *
*                                                              *
***************************************************************/
void data_cache_off()
{

#ifdef __i960_JF__

  dcctl_control(DISABLE_DCACHE, 0, 0);

#endif

#ifdef __i960_CF__

  /* passes the MCON register to modify, and the control group *
   * in which it is contained.                                 */
  dcctl_control(DISABLE_DCACHE, (unsigned int )MCON10, 
                                    (unsigned int )MCON_8_TO_11);

#endif
}

/***************************************************************
*  File:         dcache.c                Author: Intel Corp.   *
*                                                              *
*  Procedure:    data_cache_on                                 *
*                                                              *
*  Description:                                                *
*                This procedure call the dcctl macro.  It      *
*                enables the data cache.                       *
*                                                              *
*    Inputs    - None                                          *
*    Outputs   - None                                          *
*                                                              *
***************************************************************/
void data_cache_on()
{

#ifdef __i960_JF__

  dcctl_control(ENABLE_DCACHE, 0, 0);

#endif

#ifdef __i960_CF__

  /* passes the MCON register to modify, and the control group *
   * inwhich it is contained.                                  */
  dcctl_control(ENABLE_DCACHE, (unsigned int )MCON10,   
                                    (unsigned int )MCON_8_TO_11);

#endif
  
}
