 /**************************************************************************
 * File Name:   memtst16.c                                                 *
 *                                                                         *
 *                                                                         *
 * Author:      Intel Corp.                     Date:   3/11/95            *
 *                                                                         *
 * Disclaimer:                                                    	   *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY,  	   *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR      	   *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no    	   *
 *              guarantee or representations regarding the use    	   *
 *              of, or the results of the use of, the software    	   *
 *              and documentation in terms of correctness,        	   *
 *              accuracy, reliability, currentness, or            	   *
 *              otherwise,  and you rely on the software,         	   *
 *              documentation, and results solely at your own     	   *
 *              risk.                                             	   *
 *                                                                         *
 * Description: This code will allow the user to test an area of memory    *
 *              located on the Cyclone Board.  Care must be taken to       *
 *              make sure the area under test is not being used by the     *
 *              monitor.                                                   *
 *                                                                         *
 *                                                                         *
 * Revision History:                                                       *
 * ----------------------------------------------------------------------- *
 ***************************************************************************/
typedef unsigned int UINT;      

#include <stdio.h>
#include <stdlib.h>
#include "memtest.h" 
#include "system.h"

void data_cache_off();
void dram_integ(short int *address);

/***************************************************************************
*  File Name:  memtst16.c                                                  *
*                                                                          *
* Procedure:   main                                                        *
*                                                                          *
* Author:      Intel Corp.                             Date:3/11/95        * 
*                                                                          *
*                                                                          *
*                                                                          *
* Description:                                                             *
*              This procedure performs 16-bit writes of F's and 0's        *
*              on the memory allocated, and reads the address to make sure *
*              the write was performed correctly.                          *
*                                                                          *
*                                                                          *
*                                                                          *
*  Revision History:                                                       *
*     None                                                                 *
***************************************************************************/

void main()
{
		   /* Temporary variable for test types */
unsigned int	   i   = 1,
                   /* Pointer to the starting address of free memory */ 
                   *pointer;

		   /* Temporary variable */
char               tmp;



/* Allocate free memory space to test; 3200 bytes  */
pointer = (int *)malloc (800 * sizeof(int));

if(pointer != (unsigned int *) 0)
  {

     /* Disables data cache.  This allows for the external memory to be *
      * used, and not the data cache.                                   */
     data_cache_off();


     printf("\nThis program will run a 16-bit test on the external memory.\n"); 


	       
     printf("\nTest to be implemented is %s test.\n",test_types[i]);

     /* Perform test */
     dram_integ((short int *)pointer);

     printf("\nAll tests are complete.");
  }

else
  printf("Not enough free memory to test size requested");



}  /* end main() */


/***************************************************************
*  File:         memtst16.c              Author: Intel Corp.   *
*                                                              *
*  Procedure:    data_cache_off                                *
*                                                              *
*  Description:                                                *
*                This procedure call the dcctl macro.  It      *
*                disables the data cache.                      *
*                                                              *
*    Inputs    - None                                          *
*    Outputs   - None                                          *
*                                                              *
***************************************************************/
void data_cache_off()
{

#ifdef __i960_JF__

  dcctl_control(DISABLE_DCACHE, 0, 0);

#endif

#ifdef __i960_CF__

  /* passes the MCON register to modify, and the control group *
   * inwhich it is contained.                                  */
  dcctl_control(DISABLE_DCACHE, (unsigned int )MCON10, 
                                             (unsigned int )MCON_8_TO_11);

#endif
}
 


/**************************************************************************
* File:        memtst16.c                          Author:  Intel Corp.   *
*                                                                         *
* Procedure:   dram_integ                                                 *
*                                                                         *
* Description: This procedure performs testX writes and reads.  It will   *
*              write to a memory location as a short(2bytes),then performs*
*              a read on that specific memory location.                   *
*                                                                         *
*       Inputs: address  starting address to be tested.                   *
*      Outputs: None                                                      *
*                                                                         * 
*  Revision History:                                                      *
*     None                                                                *
**************************************************************************/
void dram_integ(short int *address)
{
			     /* final address to be tested */
unsigned int              final_address,                                    
			     /* Counts the number of errors */
			  error_count = 0,
                             /* Size of the free memory allocated in short */
                          size = 1600, 
			     /* Variable controlling tests */                   
                          test = 1;

                             /* pointer to address under test */
short int                 *addr;

		             /* Temporary variable */ 
register		  tmp;



  

   /* Initial address to be tested, 16 bit test */
   addr = (short int *)address;


   /* Final address to be tested */
   (unsigned int *)final_address = addr + size;

   /* Begin memory check */
   printf("Starting address = %x\n",addr);
   printf("Ending address   = %x\n\n",final_address);
   printf("\nPress enter to begin test with 0's. ");
   tmp = getchar();

   while(addr!=(short int *)(final_address))
   {
      /* Assign value to address */
      *addr = bit_0type[test];

      /* Check if read correct data */
      if((*addr & bit_ftype[test]) != bit_0type[test])
	{
	  /* If error occurred, print data */                    
	  printf("Error occurred at address %x.\n",addr);

	  printf("Expected data is %x.\n",bit_0type[test]);

	  printf("Actual data is %x.\n\n",*addr);

	  error_count++;

	  printf("\nPress enter to continue. ");
          tmp = getchar();
	}

    
      /* Increment address by 2 bytes */
      addr = addr + 1;

      /* If more than 20 errors occurred, stop the test.    */
      if(error_count > 20) 
	{
	   printf("More than 20 errors occurred, test terminated.\n");
	   printf("\nPress enter to continue with F's test. ");
           tmp = getchar();
	   break;
	}



   } /* end of while loop  */

  printf("Number of errors that occurred is %d\n\n",error_count);

 

 
  /* Reinitialize error_count for 0's test */
  error_count = 0;

   
  /* Reinitial address to be tested, 16 bit test */
  addr = (short int *)address;
 

  /* Begin test for F's */
  printf("Begin test for f's.\n");
  printf("\nPress enter to continue. ");
  tmp = getchar();

  while(addr!=(short int *)(final_address))
    {
       
 
      /* Assign value to address */
      *addr = bit_ftype[test];
 

      /* Check if errors occurred */ 
      if((*addr & bit_ftype[test]) != bit_ftype[test])
	{   
	  /* Print  errors */
	  printf("Error occurred at address %x.\n",addr);
		
	  printf("Expected data is %x.\n",bit_ftype[test]);
		 
	  printf("Actual data is %x.\n\n",*addr);
	     
	  error_count++;
		    
	  printf("\nPress enter to continue. ");
          tmp = getchar();
		    
	}
	      
      
      /* Increment address by 2 bytes */
      addr = addr + 1;
     
      /* If more than 20 errors occurred, stop the test.    */
      if(error_count > 20)
	{
	  printf("More than 20 errors occurred, test terminated.\n");
	  printf("Press enter to continue with next test. ");
          tmp = getchar();
	  break;
	}
						      
    }  /* end of while loop  */

  printf("Number of errors that occurred is %d\n\n",error_count);

}/* End dram_quad_integ */


