/****************************************************************
 *                                                              *
 * File: system.c                       Date: 12/13/94          *
 *                                                              *
 * Disclaimer:                                                  *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR    *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no  *
 *              guarantee or representations regarding the use  *
 *              of, or the results of the use of, the software  *
 *              and documentation in terms of correctness,      *
 *              accuracy, reliability, currentness, or          *
 *              otherwise,  and you rely on the software,       *
 *              documentation, and results solely at your own   *
 *              risk.                                           *
 *                                                              *
 * Description:                                                 *
 *              The purpose of this file is to provide the      *
 *              functions necessary to issue a JX or CX         *
 *              Microprocessor specific instruction not         *
 *              assessable through 'C'.                         *
 *                                                              *
 * Note: For further information on the  instructions,          *
 *       consult your i960(R) Microprocessor User Manual.       *
 *                                                              *
 * Revision History:                                            *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/
 #include "system.h"


/****************************************************************
 *                                                              *
 * File: system.c                       Author: Intel Corp.     *
 *                                                              *
 * Procedure: i960_modpc                Date: 12/13/94          *
 *                                                              *
 *                                                              *
 * Description:                                                 *
 *              This routine allows the user to issue the       *
 *              modpc assembly language instruction.  The       *
 *              inputs are the PC mask value and the data to    *
 *              be written to the PC register. No results are   *
 *              returned.  This code is applicable to both      *
 *              the JX and CX Microprocessors.                  *
 *                                                              * 
 *    Inputs  - mask   bits to mask                             *
 *              data   value to insert into pc register         *
 *    Outputs - old pc value                                    *
 *                                                              *
 *                                                              *
 ****************************************************************/
unsigned int i960_modpc(unsigned int mask, unsigned int data)
{   /* begin i960_modpc() */

__asm__ __volatile__("modpc %1,%1,%0" : "=d"(data) : "dI"(mask), "0"(data)); 

}   /* end i960_modpc() */



/****************************************************************
 *                                                              *
 * File: support.c                      Author: Intel Corp.     *
 *                                                              *
 * Procedure: i960_sysctl               Date: 12/13/94          *
 *                                                              *
 *                                                              *
 * Description:                                                 *
 *              This routine allows the user to issue the       *
 *              sysctl assembly language instruction.  The      *
 *              inputs are the sources 1, 2, and 3 as           *
 *              described in the i960(R) Microprocessor User    *
 *              Manual.  This code is applicable to both        *
 *              the JX and CX Microprocessors.                  *                                         
 *                                                              *
 *                                                              * 
 *    Inputs  - src1                                            *
 *              src2                                            *
 *              src3                                            *
 *    Outputs - None                                            *
 *                                                              *
 ****************************************************************/
void i960_sysctl(unsigned int src1, void (*src2)(), void *src3)
{   /* begin i960_sysctl() */

   __asm__ __volatile__("sysctl %0,%1,%2": :"d"(src1), "d"(src2), "d"(src3));

}   /* end i960_sysctl() */



/***************************************************************
*  File:        system.c              Author: Intel Corp.      *
*                                                              *
*  Procedure:   i960_dcctl                                     *
*                                                              *    
*  Description:                                                *
*               Enables or disables the data cache.  MCON10    *
*               corresponds to the dram region.                *
*                                                              *            
*  Inputs:      src1, src2, and src3 as defined in the         *
*               instruction set reference for the JX.          *
*                                                              *
*               CX definitions are as follows:                 *
*               src1 = enable or disable data cache.           *
*               src2 = MCON register to modify. (use MCON10    *
*                      if using cyclone board.)                *
*               src3 = register control group.                 *
*  Outputs:     None                                           *
*                                                              *
****************************************************************/
void i960_dcctl(unsigned int src1, unsigned int src2, unsigned int src3)
{

   #ifdef __i960_JF__   

      __asm__ __volatile__("dcctl %0,%1,%2": :"d"(src1), "d"(src2), "d"(src3));

   #endif

   #ifdef __i960_CF__
 
       /* Variable declaration */        

       unsigned int    /* Old value of MCON register */
                       old_value,

                       /* New value of MCON register */
                       new_value;
       
                       /* Pointer to prcb address  */
       PRCB            *ptr_to_prcb;
		
		       /* Pointer to control table address   */
       CONTROL_TABLE   *ptr_to_cntrl_tbl;     
      

   
    /* Get prcb address */
    ptr_to_prcb = (PRCB *)get_prcb_adr();

    /* Pointer to control table */
    ptr_to_cntrl_tbl = ptr_to_prcb->cntrl_table_adr;

    /* Get old value of MCON registers */
    old_value = ptr_to_cntrl_tbl -> control_reg[src2];	

    /* Check if enable or disable data cache */
    if(src1)
           /* Enable data cache */
      	new_value = (0x800000 | (old_value & ~0x800000));
     
    else  /* Disable data cache */
      	new_value =  old_value & ~0x800000;  
    
                
    /* Place new value into MCON registers */   
    ptr_to_cntrl_tbl -> control_reg[src2] = new_value;

  
    /* Store MCON10 register */
    system_control(LOAD_CONTROL_REG, src3 ,  0, 0, 0);
  
   #endif

}


/****************************************************************
 *                                                              *
 * File:        system.c                Author: Intel Corp.     *
 *                                                              *
 * Procedure:   interrupt_enable        Date: 3/11/95           *
 *                                                              *
 *                                                              *
 * Description:                                                 *
 *              This routine allows the user to enable the      *
 *              global interrupts.  The JX processor uses the   *
 *              "inten" command while the CX modifies the       *
 *              icon register.                                  *
 *                                                              * 
 *    Inputs  - None                                            *
 *    Outputs - None                                            *
 *                                                              *
 *                                                              *
 ****************************************************************/
void interrupt_enable()
{
            /* temprorary variable */
register    tmp;


   #ifdef __i960_CF__  /* write to icon register bit 10 */
  
      CONTROL_TABLE  *ptr;          /* pointer to control table */
      PRCB           *ptr_to_prcb;  /* pointer to prcb */

                     /* old value of icon register */      
      unsigned int   old_value,
                     /* new value of icon register with *
                      * global enable bit set.          */
                     new_value;



          /* get pointer to prcb */
         ptr_to_prcb = (PRCB *)get_prcb_adr();

          /* get pointer to control register */
         ptr =  ptr_to_prcb->cntrl_table_adr;
 
         /* modify global enable bit */
         old_value = ptr -> control_reg[ICON];
         new_value = (old_value & 0xfbff);
         ptr -> control_reg[ICON] = new_value;

   #endif

   #ifdef __i960_JF__    /* specific JX enable command */

        __asm__ __volatile__("intctl 1, %0":  : "dI"(tmp));

   #endif

}


/*******************************************************************
*                                                                  *
*  File:         system.c               Author:  Intel Corp.       *
*                                                                  *
*  Procedure:    modify_dmac                                       *
*                                                                  *
*                                                                  *
*  Description:                                                    *
*                This procedure will perform one of two functions. *              
*                                                                  *
*                1)  This routine will set the the specified       *
* 		     bit if set = 1.                               *
*                                                                  *
*                2)  This routine will clear the specified bit     *
*                    if set = 0.                                   *
*                                                                  *
*                The CX Microprocessor only performs these         *
*                functions.                                        *
*                                                                  *
*                                                                  * 
*    Inputs  - set    specifies whether to clear or set a bit in   *
*                     the DMAC register.                           *
*              bit    bit to modify in the DMAC register.          *                             *
*    Outputs - None                                                *
*                                                                  *
*******************************************************************/
#ifdef __i960_CF__

   void modify_dmac(unsigned int set, unsigned int bit)
   {

      if(set)
            /* Set bit in DMAC register */
         asm("setbit %0, sf2, sf2": : "dI" (bit));

      else
            /* Clear bit in DMAC register */
         asm("clrbit %0, sf2, sf2": : "dI" (bit));

   }
#endif

 
/*******************************************************************
*                                                                  *
*  File:         system.c               Author:  Intel Corp.       *
*                                                                  *
*  Procedure:    i960_mod_reg                                      *
*                                                                  *
*                                                                  *
*  Description:                                                    *
*                This procedure will perform one of 3 functions.   *              
*                                                                  *
*                1)  This routine will set the IMSK register       *
* 		     to accept external interrupts from bit 6.     *
*                                                                  *
*                2)  This routine will clear the IPND register.    *
*                                                                  *
*                3)  This routine will configure the DMAC          *
*                    register. (only applicable in CX)             *
*                                                                  *
*                The CX and JX Microprocessors perform these       *
*                functions in a different manner.                  *
*                                                                  *
*                                                                  * 
*    Inputs  - reg     register to modify (IMSK, DMAC or IPND)     *
*              value   value to put into register                  *                             *
*    Outputs - None                                                *
*                                                                  *
*******************************************************************/

void i960_mod_reg (int reg, unsigned int value)
{
          
  #ifdef __i960_CF__       

         /* Modify IMSK or IPND Registers for CX*/
      
        if (reg == 0)
             {
                 /* Modify IPND register */
                value = 0x00000000 | value;
		asm("mov %0,sf0": : "dI" (value));
             }
            
	else if (reg  == 1)
             {
                 /* Modify IMSK register */
                value = 0x00000000 | value;
		asm("mov %0,sf1": : "dI" (value));
             }

        else if (reg == 2)
             {
                 /* Modify DMAC register */
                value = 0x00000000 | value;
                asm("mov %0,sf2": : "dI" (value));
             }
              
        else  
                printf("Error in modifing register.\n");
  #endif  

  #ifdef __i960_JF__                       /* Modify IMSK or IPND Registers for JX */
      
     register int mmr, tmp;              /* Temporary variable declaration */
  
        if (reg == 0)        
              {
                /*   memory-mapped address of IPND register */
                asm("lda     0xff008500, %0"   : "=&d" (mmr)); 
                asm("lda     0x30ff, %0"   : "=&d" (tmp));   /* bits to mask */      
                asm("atmod   %1, %2, %0 "   :  : " dI" (value), "dI" (mmr), "dI"  (tmp));
              }


        else if (reg == 1)
              {
                /* memory-mapped address of IMSK register */ 
                /* modifies IMSK register in dedicated mode */          
                asm("lda     0xff008504, %0"   : "=&d" (mmr)); 
                asm("lda     0x30ff, %0"         : "=&d" (tmp));  /* bits to mask */
                asm("atmod   %1, %2, %0"    :  : " dI" (value), "dI" (mmr), "dI"  (tmp));
              }

         else
             printf("Error in modifing register. \n");
        
  #endif
}
 
 

