 /******************************************************************
 *                                                                 *
 * File:        dma.c                      Date: 3/11/95           *
 *                                                                 *
 * Disclaimer:                                                     *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY,   *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR       *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no     *
 *              guarantee or representations regarding the use     *
 *              of, or the results of the use of, the software     *
 *              and documentation in terms of correctness,         *
 *              accuracy, reliability, currentness, or             *
 *              otherwise,  and you rely on the software,          *
 *              documentation, and results solely at your own      *
 *              risk.                                              *
 *                                                                 * 
 * Overview:                                                       *
 *                                                                 *
 *  This code provides an example of programming the DMA           *
 *  Controller of the 80960 CX microprocessor.  This example       *
 *  is setup for block mode chaining transfer.                     *
 *                                                                 *
 * Revision History:                                               *
 * ------------------------------------------------------------    *
 *******************************************************************/
#include <stdio.h>
#include "dma.h"
#include "system.h"

extern volatile unsigned int buffer;
  


/*******************************************************************
 * File:       dma.c                       Author:  Intel Corp.    *
 *                                                                 *
 * Procedure:  Main                                                *
 *                                                                 *
 * Description:                                                    *
 *                                                                 *
 *      This code will disable the dma controller and then         *
 * configure it.  Channel 0 will be setup for use                  *
 * while the other channels will be disabled.  The DMAC register   *
 * will then be setup accordingly.  The DMA Controller will then   *
 * be enabled.  At the end of a transfer, an interrupt will be     *
 * generated, and the interrupt routine will clear the channel     *
 * wait bit in the DMAC register allowing the next descriptor to   *
 * be read. This will continue until a NULL pointer is encounter   *
 * in the next pointer of the descriptor.                          *
 *                                                                 *
 *******************************************************************/

main()
{
                       /* temporary variable */
register               tmp;
                       /* dma transfer is complete */
volatile unsigned int  done = 0;
 
  /* 1.  Modify the DMAC register                              */
 
         i960_mod_reg(DMAC, 0x200000);
 
  /* 2.  Setup channel 0                                       */
       
         setup_sdma();
        
         
  /* 3.  Setup for dma interrupt handling                      */

         setup_interrupt();

 
  /* 4.  Enable dma                                            */

         modify_dmac(SET, 0);

  /* 5.  Wait for the DMA to complete.   		       */
  /*     check done bit in DMAC register                       */

         while(!done)
           {
             asm("mov sf2, %0": "=&d"(tmp));
             if((tmp & 0x1000))
               done = 1;
            }



}

