/*******************************************************************
 *                                                                 *
 * File:        int_rout.c                 Date: 3/11/95           *
 *                                                                 *
 * Disclaimer:                                                     *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY,   *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR       *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no     *
 *              guarantee or representations regarding the use     *
 *              of, or the results of the use of, the software     *
 *              and documentation in terms of correctness,         *
 *              accuracy, reliability, currentness, or             *
 *              otherwise,  and you rely on the software,          *
 *              documentation, and results solely at your own      *
 *              risk.                                              *
 *                                                                 * 
 * Overview:                                                       *
 *                                                                 *
 *              This code will provide the interrupt DMA           *
 *              handling routine.                                  *
 *                                                                 *
 * Revision History:                                               *
 * ----------------------------------------------------------------*
 *******************************************************************/

#include "system.h" 


/* Keeps count of which buffer transfer has completed */   
unsigned volatile int buffer = 1;

/* function declarations */
void interrupt_service_routine(void);

 

/*******************************************************************
 * File:       int_rout.c                  Author:  Intel Corp.    *
 *                                                                 *
 * Procedure:  setup_interrupt                                     *
 *                                                                 *
 * Description:                                                    *
 *                                                                 *
 *     This code will perform the following steps in handling of   *
 * a dma interrupt.                                                *
 *        1.  modify the ICON register                             *
 *        2.  modify the IMAP register                             *
 *        3.  caching of vector and interrupt handling procedure   *
 *        4.  lower processor priority                             *
 *        5.  modify the IMSK register                             *
 *        6.  clear the IPND register                              *
 *        7.  enable global interrupts                             *
 *                                                                 *
 *                                                                 *
 *******************************************************************/
void setup_interrupt()
{ 
                       
                        /* Pointer to prcb address                 */
    PRCB              *ptr_to_prcb;

                        /* Pointer to interrupt table address      */
    INTERRUPT_TABLE   *ptr_to_int_tbl;

                        /* Pointer to control table  address       */
    CONTROL_TABLE     *ptr_to_cntrl_tbl;

                        /* Pointer to internal data ram memory     */
    unsigned int      *dram = (unsigned int *)0x0,
                        /* Pointer to ISR */
                      *proc_addr ;
     
 

/*  1. Get the pointer to the PRCB address.                               */

       
          ptr_to_prcb = get_prcb_adr();
          
   
/*  2. Get the pointer to the interrupt table and control table.          */

          ptr_to_int_tbl = ptr_to_prcb->interrupt_table_adr;
          ptr_to_cntrl_tbl = ptr_to_prcb->cntrl_table_adr;
        
/*  3. Set the ICON register:                                             */
/*     Dedicated interrupt, falling edge activated, global                */
/*     interrupts disabled, mov to r3 mask unchanged, fetch               */
/*     vector enable fast, fast sampling, DMA suspend on Interrupt.       */       
 
       ptr_to_cntrl_tbl -> control_reg[ICON] = 
              (ptr_to_cntrl_tbl -> control_reg[ICON] & 0xffff0000) | 0xe7fc;
 
        
/*  4. Set the IMAP2 register:                                            */
/*     vector number E2 (226) for DMA interrupt pin                       */
  
       ptr_to_cntrl_tbl -> control_reg[IMAP2] = 
                (ptr_to_cntrl_tbl -> control_reg[IMAP2] & 0xfff0) | 0x000e;
 
/*  5. Store the vector number in internal data ram memory                */ 
/*     Last 2 bits must end with 10 inorder to be cached properly.        */   
       
       dram[0xe] = (unsigned int)interrupt_service_routine | (unsigned int)0x2; 
   
/*  6. Load and lock the interrupt procedure in the instruction           */ 
/*     cache.                                                             */

       proc_addr = (unsigned int *)interrupt_service_routine;
       system_control(CONFIGURE_ICACHE, LOCK_HALF_ICACHE, 0,
                                                  (void *) proc_addr ,0);
        
/*  7. Store control registers with sysctl.                               */
       
       system_control(LOAD_CONTROL_REG, INTERRUPT_GROUP, 0, 0, 0);
 
/*  8. Change processor priority to 0 so that our interrupt               */
/*     which is at priority 28 will be processed                          */

       modify_pc_priority(PRIORITY0);


/*  9. Set IMSK register to accept DMA interrupts on channel              */
/*     zero                                                               */
  
       i960_mod_reg(IMSK,0x100);

 
/* 10. Clear the interrupt pending register IPND                          */
    
       i960_mod_reg(IPND,0); 
       
/* 11. Enable global interrupts                                           */

       interrupt_enable(); 
 
/* 12. Reload the icon register with the new value                        */
 
       system_control(LOAD_CONTROL_REG, INTERRUPT_GROUP, 0, 0, 0);
 

}   /* end interrupt_routine */      
 
/***************************************************************
*  File:         int_rout.c              Author: Intel Corp.   *
*                                                              *
*  Procedure:    interrupt_service_routine                     *
*                                                              *
*  Description:                                                *
*                This code will display when one transfer      *
*                has occurred.  It will clear the channel      *
*                wait bit allowing the chained transfer to     *
*                continue.                                     *
*                                                              *
*    Inputs    - None                                          *
*    Outputs   - None                                          *
*                                                              *
***************************************************************/
void interrupt_service_routine()
{  


   /* Clear channel wait bit so next descriptor can be read */
   modify_dmac(CLR, 16);


   /* Displays when each buffer transfer is complete and increment */
   printf("Buffer transfer #%d is complete.\n\n",buffer++);

}
