 
/*****************************************************************************
 *							                     *
 * Copyright (c) 1993, 1994 Intel Corporation                                *
 *									     *
 * Intel hereby grants you permission to copy, modify, and distribute this   *
 * software and its documentation.  Intel grants this permission provided    *
 * that the above copyright notice appears in all copies and that both the   *
 * copyright notice and this permission notice appear in supporting          *
 * documentation.  In addition, Intel grants this permission provided that   *
 * you prominently mark as "not part of the original" any modifications      *
 * made to this software or documentation, and that the name of Intel        *
 * Corporation not be used in advertising or publicity pertaining to         *
 * distribution of the software or the documentation without specific,       *
 * written prior permission.                                                 *
 *									     *
 * Intel Corporation provides this AS IS, WITHOUT ANY WARRANTY, EXPRESS OR   *
 * IMPLIED, INCLUDING, WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY   *
 * OR FITNESS FOR A PARTICULAR PURPOSE.  Intel makes no guarantee or         *
 * representations regarding the use of, or the results of the use of,       *
 * the software and documentation in terms of correctness, accuracy,         *
 * reliability, currentness, or otherwise; and you rely on the software,     *
 * documentation and results solely at your own risk.                        *
 *									     *
 * IN NO EVENT SHALL INTEL BE LIABLE FOR ANY LOSS OF USE, LOSS OF BUSINESS,  *
 * LOSS OF PROFITS, INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES   *
 * OF ANY KIND.  IN NO EVENT SHALL INTEL'S TOTAL LIABILITY EXCEED THE SUM    *
 * PAID TO INTEL FOR THE PRODUCT LICENSED HEREUNDER.                         *
 *    									     *
 ****************************************************************************/
    

    .file    "asm_flt.s"


 
 /****************************************************************************
 * File Name:   asm_flt.s                                                    *
 *                                                                           *
 * Procedure:   parafault()                                                  *
 *                                                                           *
 * Author:      Intel Corp.                         Date:   3/11/95          *
 *                                                                           *
 * Description: This procedure produces a parallel fault by creating 2       *
 *              faults which are executed in parallel.                       *
 *                                                                           *
 * Parameters:  None                                                         *
 *                                                                           *
 * Returns:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
    .globl    _parafault

 _parafault: 
    
     /* clear the No Imprecise Faults bit and Integer Overflow flag bits */
     lda   0x9000, r3
     modac r3,0, g0

     b     _muli_addi

   .align 4

_muli_addi:

   /*  instructions will cause overflow fault, 2 faults generated since  *
    *  instructions will execute in parallel                             */
     lda   0x7fffffff, g8
     lda   0x1, g9
     lda   0x7fffffff, g2
     lda   0x2, g4
    muli g2,g4,g6
    addi g8,g9,g10
    ret

 
 /****************************************************************************
 * File Name:   asm_flt.s                                                    *
 *                                                                           *
 * Procedure:   protfault()                                                  *
 *                                                                           *
 * Author:      Intel Corp.                         Date:   3/11/95          *
 *                                                                           *
 * Description: This procedure creates a protection fault by making a call   *
 *              beyond the range of the table.                               *
 *                                                                           *
 * Parameters:  None                                                         *
 *                                                                           *
 * Returns:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
    .globl    _protfault

_protfault:

    lda   260,r12

  len:  calls r12


 /****************************************************************************
 * File Name:   asm_flt.s                                                    *
 *                                                                           *
 * Procedure:   super_mode                                                   *
 *                                                                           *
 * Author:      Intel Corp.                         Date:   3/11/95          *
 *                                                                           *
 * Description: This procedure places the processor into supervisor mode.    *
 *              This is needed since the Type fault puts the processor into  *
 *              user mode.  This will allow for re-running of the program    *
 *              without any glitches.                                        *
 *                                                                           *
 * Parameters:  None                                                         *
 *                                                                           *
 * Returns:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
     .globl   _super_mode

_super_mode:

    andnot 0xf, pfp, pfp

/*****************************************************************************
 * File          asm_flt.s                           Date: 3/11/95           *
 *                                                                           *
 * Procedure:    FAULT_RECORD *i960_get_fault_rec(void)                      *
 *                                  					     *
 * Description:  Called from within a fault handler this function            *
 *               calculates the address of the fault record, and returns     *
 *               that address.  					     *
 *               WARNING: This function MUST be called from the "base"       *
 *               level of the fault handler.  It can NOT be nested.          * 
 *               This is because the function relies on the value of         *
 *               the frame pointer (fp).                                     *
 * 									     *
 * Inputs:       None                                                        *
 * Outputs:      pointer to a fault record 				     *
 ****************************************************************************/

    .set FAULT_RECORD_SIZE, 16
    .globl _i960_get_fault_rec

_i960_get_fault_rec:

    andnot    0xf, pfp, g0
    lda       -FAULT_RECORD_SIZE(g0), g0
    ret


 
