 /*********************************************************************
 *                                                                    *
 * File:        fault.c                 Date: 3/11/95                 *
 *                                                                    *
 * Disclaimer:                                                        *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY       *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR          *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes           * 
 *              no guarantee or representations regarding the         * 
 *              use of, or the results of the use of, the             * 
 *              software and documentation in terms of                * 
 *              correctness, accuracy, reliability,                   * 
 *              currentness, or otherwise, and you rely on the        * 
 *              software, documentation, and results solely           * 
 *              at your own risk.                                     *
 *                                                                    *
 * Description:                                                       *
 *                                                                    *
 *  This program shows the steps taken in setting up the fault        *
 *  handling procedures in the fault and system procedure table.      *
 *  Also, the steps required to recover from a fault.                 *
 *                                                                    *        
 *                                                                    *
 * Revision History:                                                  *
 * ------------------------------------------------------------------ *
 *                                                                    *
 *********************************************************************/
 /*********************************************************************
 * File Name:   fault.c                                               *
 *                                                                    *
 * Procedure:   main                                                  *
 *                                                                    *
 * Author:      Intel Corp.          Date:   3/11/95                  *
 *                                                                    *
 * Description:                                                       *
 *              This program will generate the following faults:      *
 *                  arithmetic,                                       *
 *                  constraint,                                       *
 *                  operation,                                        *     
 *                  protection,                                       *
 *                  parallel,                                         *
 *                  type                                              *
 *              Handling procedure are called either through a local  *
 *              call or a system call.  The handling procedure, when  *
 *              needed,will calculate the RIP and/or perform other    *
 *              actions allowing for a continuing program execution.  *
 *                                                                    *
 * Parameters:  None                                                  *
 *                                                                    *
 * Returns:     None                                                  *
 *                                                                    *
 * Revision History:                                                  *
 * ------------------------------------------------------------------ *
 **********************************************************************/
typedef unsigned int UINT;

#include <stdio.h>
#include "system.h"
#include "fault.h"

 
main()
{
   register   tmp;



/* Load faults into fault and system procedure tables.      */

   load_flt_proc();  
       

/*  Set the processor into supervisor mode.                 */
    
    asm("calls   2"); 


/* Generate faults   */



    /* Generate an arithmetic fault                         */
    /* Divide by zero will generate an arithmetic fault     */
      __asm__ __volatile__("divo  0,2,%0":   "=&d" (tmp));
   

    /* Generate a constraint fault                           */
       /* modify condition code bits to 000 */
          __asm__ __volatile__("modac  0x7,0x0,%0":  : "dI" (tmp));
    
       /* fault if AC register cc bits = 000 */
          __asm__ __volatile__("faultno");
    
   /* Generate a operation fault                           */
   /*  Write to an unaligned register                       */
      __asm__ __volatile__("ldt 0,r3");


   /* Generate a protection fault */
   /* call assembly program, producing protection fault */
      protfault();
   

   /* Generate a parallel fault */
   /* call assembly routine */
      parafault();
   

   /* Generate a type fault        */
      /* load constant into variable */
         __asm__ __volatile__("ldconst 0x0, r4");

      /* change processor to user mode */
         __asm__ __volatile__("modpc 2, 2, r4");

      /* attempt to write to modpc */
         __asm__ __volatile__("modpc 2, 2, r4");
    


   printf("\nAll faults have been generated and handled.\n");  
 
}   /* end main() */

 
 

/************************************************************************
 * File Name:   fault.c                                                 *
 *                                                                      *
 * Procedure:   load_flt_proc                                           *
 *                                                                      *
 * Author:      Intel Corp.          Date:   3/11/95                    *
 *                                                                      *
 * Description:                                                         *
 *              This procedure will place the fault handling procedures *
 *              into the fault table and/or the system procedure table. *
 *                                                                      *
 *      Inputs:  None                                                   *
 *     Outputs:  None                                                   *
 *                                                                      *
 * Revision History:                                                    *
 * ---------------------------------------------------------------------*
 ************************************************************************/
void load_flt_proc()
{
               
 /* Set processor into supervisor mode with a supervisor call */
 mod_sysproc_entry(ENTRY2, (unsigned int)super_mode, 1);
 


 /* load the arithmetic procedure as local call */ 
 mod_fault_table(ARITHMETIC, (unsigned int)arith_proc, 0);

 /* load Type fault as local call */
 mod_fault_table(TYPE, (unsigned int )type_proc, 0);
         
 /* load Constraint fault as local call */
 mod_fault_table(CONSTRAINT, (unsigned int )const_proc, 0);

 /* load Protection fault as local call */
 mod_fault_table(PROTECTION, (unsigned int )prot_proc, 0);

 /* load Operation fault as system call */
 mod_fault_table(OPERATION, (unsigned int )1, 1);
 mod_sysproc_entry(ENTRY1, (unsigned int )oper_proc, 0);

 /* load Parallel fault as system call */
 mod_fault_table(PARALLEL, (unsigned int )3, 1); 
 mod_sysproc_entry(ENTRY3, (unsigned int )para_proc, 0);
}
   
  
/*************************************************************************
* File:        fault.c                                                   *
*                                                                        *
* Procedure:   mod_fault_table                                           *
*                                                                        *
* Description: Modifies an entry in the Fault Table                      *
*                                                                        *
* Inputs:                                                                *
*    entry_offset: offset of the desired Fault Entry (0 = Parallel,      *
*                  2  = Trace, 4 = Operation, 6 = Arithmetic,            *
*                  10 = Constraint, 14 = Protection, 20 = Type)          *
*    new_value: system_call_sel = 0: Fault-Handler Procedure Address     *
*               system_call_sel = 1: Fault-Handler System Procedure      *
*                                    Number                              *
*    system_call_sel: = 0 to select a Local-Call for the entry; bits     *
*                       0 & 1 of the entry and the second word of the    *
*                       entry are set to 0                               *
*                     = 1 to select a System-Call for the entry; bits    *
*                       0 & 1 of the entry are set to 10, and the second *
*                       word of the entry is set to 27FH                 *
* Outputs:                                                               *
*     NONE                                                               *
*************************************************************************/
void mod_fault_table (unsigned int entry_offset, unsigned int new_value,
                                                     int system_call_sel)
{
                  /* fault table address */
    FAULT_TABLE   *fault_table;
 
                  /* prcb address */ 
    PRCB          *prcb;

      /* get prcb address */
      prcb = get_prcb_adr();

      /* get fault table address */ 
      fault_table = prcb->fault_table_adr;
     
      /* if system call, place 10 in final bits */ 
      if(system_call_sel)
        {
           new_value = new_value << 2;
           new_value = new_value + 2;
        }
 
      /* place value into fault table */ 
      fault_table->fault_entry[entry_offset] = new_value; 

      /* if system call, place 0x27f in next location */
      fault_table->fault_entry[entry_offset + 1]= 
                                           0x27f * (system_call_sel !=0); 

 

         

} /* end mod_fault_table() */


/*************************************************************************
* File:        fault.c                                                   *
*                                                                        *
* Procedure:   mod_sysproc_entry                                         *
*                                                                        *
* Description: Modifies an entry in the System Procedure Table           *
*                                                                        *
* Inputs:                                                                *
*    entry_num:   System Procedure Number                                *
*    sysproc_add: System Procedure Address                               *
*                 0 & 1 of the entry must be set to 0 (Address must be   *
*                 word aligned)                                          *
*    sup_sel: =   0 to select a Local-Call for the procedure; bits 0 & 1 *
                  of the entry are set to 00                             *
*             =   1 to select a Supervisor-Call for the entry; bits      *
*                 0 & 1 of the entry are set to 10                       *
* Outputs:        NONE                                                   *
*                                                                        *
*************************************************************************/
void mod_sysproc_entry (unsigned int entry_num, unsigned int sysproc_add, 
                        unsigned int sup_sel)
{

                 /* pointer to system procedure table */
SYS_PROC_TABLE   *sysproc_table;

                 /* pointer to prcb */
PRCB             *prcb;

                 /* value to be placed into table */ 
unsigned int     new_entry_value = 0;

    /* get prcb address */ 
    prcb = get_prcb_adr();

    /* get system procedure table address */
    sysproc_table =  prcb->sys_proc_table_adr;
   
    /* assign procedure address to variable */
    new_entry_value = sysproc_add;

    /*  add 2 if supervisor call */
    new_entry_value = new_entry_value + (2 * (sup_sel != 0));

    /* place new value into table entry */ 
    sysproc_table->sys_entry[entry_num] = new_entry_value;
 


} /* end mod_sysproc_entry() */
 


 
