
/*****************************************************************************
 *                                                                           *
 * File:        flt_proc.c                                                   *
 *                                                                           *
 * Author:      Intel Corp.                       Date:   3/11/95            *
 * Disclaimer:                                                               *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY              *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR                 *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes                  * 
 *              no guarantee or representations regarding the                * 
 *              use of, or the results of the use of, the                    * 
 *              software and documentation in terms of                       * 
 *              correctness, accuracy, reliability,                          * 
 *              currentness, or otherwise, and you rely on the               * 
 *              software, documentation, and results solely                  * 
 *              at your own risk.                                            *
 *                                                                           *
 * Description: This file contains the fault handling procedures for each of *
 *              faults that is generated.                                    *
 *                                                                           * 
 *                                                                           * 
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *                                                                           *
 *****************************************************************************/
 #include "system.h"
 #include "flt_proc.h"
 


 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   arith_proc()                                                 *
 *                                                                           *
 * Author:      Intel Corp.                    Date:   3/11/95               *
 *                                                                           *
 * Description: This procedure contains the fault handling procedures for    *
 *              the arithmetic fault.                                        *
 *                                                                           *
 * Inputs:      None                                                         *
 *                                                                           *
 * Outputs:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void arith_proc( )
{
               /* temporary variable */
int            tmp; 

               /* pointer to fault record */
FAULT_RECORD   *fault_ptr;



  /* get location of the fault record */
  (FAULT_RECORD*)fault_ptr = (FAULT_RECORD *)i960_get_fault_rec();
  
  printf("Arithmetic handling procedure has been called.\n");
 
  /* print fault record data */ 
  fault_record_data(fault_ptr );
 
  printf("\nPress ENTER to continue ");
  tmp = getchar();                                                 


}

   



 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   oper_proc()                                                  *
 *                                                                           *
 * Author:      Intel Corp.          Date:   3/11/95                         *
 *                                                                           *
 * Description: This procedure contains the fault handling procedure for     *
 *              the operation fault.  It will modify the RIP.                *
 *                                                                           *
 * Inputs:      None                                                         *
 *                                                                           *
 * Outputs:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void oper_proc()
{

               /* temporary variables */
int            tmp;
unsigned long  tmp1;

               /* pointer to fault record */
FAULT_RECORD   *fault_ptr;



  /* get location of the fault record */
  (FAULT_RECORD*)fault_ptr = (FAULT_RECORD *)i960_get_fault_rec();

  printf("Operation handling procedure has been called.\n\n");
 
 
  /* get fault record information */
  fault_record_data(fault_ptr );
 
      
  printf("\nPress ENTER to continue ");
  tmp = getchar();                                                 
                                                           
  /* This fault type does not define the RIP.  Therefore,      *
   * before returning to the calling procedure we must set the *
   * RIP to point to the next instruction following the	       *
   * faulting instruction.				       */
  asm("flushreg");
  tmp1 = (unsigned long)(fault_ptr->addr + 1);
  asm("mov pfp, r5");
  asm("andnot 0xf, r5, r5");
  asm("st %0, 8(r5)"::"dI"(tmp1)); 

}


 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   type_proc()                                                  *
 *                                                                           *
 * Author:      Intel Corp.          Date:   3/11/95                         *
 *                                                                           *
 * Description: This file contains the fault handling procedures for the     *
 *              type fault.  If required, it will modify the RIP.            *
 *                                                                           *
 * Inputs:      None                                                         *
 *                                                                           *
 * Outputs:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void type_proc()
{
               /* temporary variable */
int            tmp; 
unsigned long  tmp1;

               /* pointer to fault record */
FAULT_RECORD   *fault_ptr;



  /* get location of the fault record */
  (FAULT_RECORD*)fault_ptr = (FAULT_RECORD *)i960_get_fault_rec();

  
  printf("Type handling procedure has been called.\n\n");
 
 
  /* get fault record information */
  fault_record_data(fault_ptr );
 
       
  printf("\nPress ENTER to continue ");
  tmp = getchar();                                                 
 
                                                     
  /* This fault type does not define the RIP.  Therefore,      */
  /* before returning to the calling procedure we must set the */
  /* RIP to point to the next instruction following the	       */
  /* faulting instruction.				       */
  asm("flushreg");
  tmp1 = (unsigned long)(fault_ptr->addr + 1);
  asm("mov pfp, r5");
  asm("andnot 0xf, r5, r5");
  asm("st %0, 8(r5)"::"dI"(tmp1)); 

}


 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   const_proc()                                                 *
 *                                                                           *
 * Author:      Intel Corp.          Date:   3/11/95                         *
 *                                                                           *
 * Description: This file contains the fault handling procedures for the     *
 *              constraint fault.  If required, it will modify the RIP.      *
 *                                                                           *
 * Inputs:      None                                                         *
 *                                                                           *
 * Outputs:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void const_proc()
{

               /* temporary variable */
int            tmp; 
unsigned long  tmp1;

               /* pointer to fault record */
FAULT_RECORD   *fault_ptr;



  /* get location of the fault record */
  (FAULT_RECORD*)fault_ptr = (FAULT_RECORD *)i960_get_fault_rec();

  
  printf("Constraint handling procedure has been called.\n\n");
 

  /* get fault record information */
  fault_record_data(fault_ptr );
 
                                                            
  printf("\nPress ENTER to continue ");
  tmp = getchar();                                                 
 
     
  /* This fault type does not define the RIP.  Therefore,      */
  /* before returning to the calling procedure we must set the */
  /* RIP to point to the next instruction following the	       */
  /* faulting instruction.				       */
  asm("flushreg");
  tmp1 = (unsigned long)(fault_ptr->addr + 1);
  asm("mov pfp, r5");
  asm("andnot 0xf, r5, r5");
  asm("st %0, 8(r5)"::"dI"(tmp1)); 

}


 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   prot_proc()                                                  *
 *                                                                           *
 * Author:      Intel Corp.          Date:   3/11/95                         *
 *                                                                           *
 * Description: This procedure contains the fault handling procedures for    *
 *              the protection fault.  If required, it will modify the RIP.  *
 *                                                                           *
 * Inputs:      None                                                         *
 *                                                                           *
 * Outputs:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void prot_proc()
{

                    /* temporary variables */
int                 tmp;
unsigned long       tmp1;

                    /* pointer to fault record */
FAULT_RECORD        *fault_ptr;



  /* get location of the fault record */
  (FAULT_RECORD*)fault_ptr = (FAULT_RECORD *)i960_get_fault_rec();
  
 
  printf("Protection handling procedure has been called.\n\n");
 
  /* get fault record information */
  fault_record_data(fault_ptr );
 
      
  printf("\nPress ENTER to continue ");
  tmp = getchar();                                                 
 
                                                                
  /* This fault type does not define the RIP.  Therefore,      */
  /* before returning to the calling procedure we must set the */
  /* RIP to point to the next instruction following the	       */
  /* faulting instruction.				       */
  asm("flushreg");   
  tmp1 = (unsigned long)(fault_ptr->addr + 1);
  asm("mov pfp, r5");
  asm("andnot 0xf, r5, r5");
  asm("st %0, 8(r5)"::"dI"(tmp1)); 



}

 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   para_proc()                                                  *
 *                                                                           *
 * Author:      Intel Corp.          Date:   3/11/95                         *
 *                                                                           *
 * Description: This procedure contains the fault handling procedures for    *
 *              the parallel fault.  It will take the greater address of the *
 *              faulting instructions and increment it.                      *
 *                                                                           *
 * Inputs:      None                                                         *
 *                                                                           *
 * Outputs:     None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void para_proc()
{

                    /* temporary variables */
int                 tmp;
unsigned long       tmp1,
                    /* address of 1st faulting instruction */
                    addr1,
                    /* address of 2nd faulting instruction */
                    addr2;

                    /* pointer to base of fault record */
unsigned int        *ptr;

                    /* pointer to fault record */
FAULT_RECORD        *fault_ptr;



  /* get location of the fault record */
  (FAULT_RECORD*)fault_ptr = (FAULT_RECORD *)i960_get_fault_rec();
  
 
  printf("\nParallel handling procedure has been called.\n\n");
 

  /* get address of both faulting instructions */
  addr1 = (unsigned long)(fault_ptr->addr);
  printf("Address of 1st faulting instruction = %x\n",addr1);

/* get address of 2nd faulting instruction  */
  /* in CX, address located 21 locations above base of fault record table */
  #ifdef __i960_CF__
      ptr = (unsigned int *)fault_ptr;
      addr2 = *(ptr - 21);
  #endif

  /* in JX, address located 13 locations above base of fault record table */
  #ifdef __i960_JF__
      ptr = (unsigned int *)fault_ptr;
      addr2 = *(ptr - 13);
  #endif

  printf("Address of 2nd faulting instruction = %x\n",addr2);


  printf("\nPress ENTER to continue ");
  tmp = getchar();                                                 

  /* find greater address of faulting instruction and increment from *
   * that address.                                                   */  
  if(addr1 > addr2)
    /* increment 1st faulting instruction */
    tmp1 = (unsigned long)(addr1 + 4);
    
  else
    /* increment 2nd faulting instruction */
    tmp1 = (unsigned long)(addr2 + 4);
  
  /* This fault type does not define the RIP.  Therefore,      */
  /* before returning to the calling procedure we must set the */
  /* RIP to point to the next instruction following the	       */
  /* faulting instruction.				       */
  asm("flushreg");
  asm("mov pfp, r5");
  asm("andnot 0xf, r5, r5");
  asm("st %0, 8(r5)"::"dI"(tmp1)); 



}


 /****************************************************************************
 * File Name:   flt_proc.c                                                   *
 *                                                                           *
 * Procedure:   fault_record_data()                                          *
 *                                                                           *
 * Author:      Intel Corp.          Date:   3/11/95                         *
 *                                                                           *
 * Description: This procedure will get the fault record and display the     *
 *              type and subtype of fault that occurred.                     *
 *                                                                           *
 * Inputs:      *fault_ptr   pointer to fault record                         *
 *                                                                           *
 * Output:      None                                                         *
 *                                                                           *
 * Revision History:                                                         *
 * ------------------------------------------------------------------------- *
 *****************************************************************************/
void fault_record_data(FAULT_RECORD  *fault_ptr)
{

                   /* type of fault that occurred */
unsigned int       type; 
                   
                   /* subtype of fault that occurred */
unsigned int       subtype;
 

  
  /* get type and subtype information */
  type =  (fault_ptr->type >> 16) & 0xff;
  subtype =   fault_ptr->type & 0xff;

  /* display information */ 
  printf("\nFault type that occurred is %x.\n",type);
  printf("Fault subtype that is %x.\n",subtype);
  printf("Faulting instruction occurred at address %x.\n",     
                                       (unsigned int)fault_ptr->addr );
}

