 /***************************************************************
 *                                                              *
 * File: system.h                         Date: 3/11/95         *
 *                                                              *
 * Disclaimer:                                                  *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR    *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes     * 
 *              no guarantee or representations regarding the   * 
 *              use of, or the results of the use of, the       * 
 *              software and documentation in terms of          * 
 *              correctness, accuracy, reliability,             * 
 *              currentness, or otherwise, and you rely on the  * 
 *              software, documentation, and results solely     * 
 *              at your own risk.                               *
 *                                                              *
 * Description:                                                 *
 *                                                              *
 *  Contains procedure's declarations and constant definitions  *
 *  used to easily issue various instructions and perform       *
 *  numerous tasks in a C program.                              * 
 *                                                              *
 *                                                              *
 * Revision History:                                            *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/

#ifndef _H_SYSTEM
#define _H_SYSTEM


/* Function declarations contained in system.c */
extern unsigned int i960_modpc(unsigned int mask, unsigned int data);
extern void i960_sysctl(unsigned int src1, void (*src2)(), void *src3);
extern void i960_dcctl(unsigned int src1, unsigned int src2, unsigned int src3);
extern void interrupt_enable();
extern void modify_dmac(unsigned int set, unsigned int bit);
extern void i960_mod_reg (int reg, unsigned int value);





    /* JX Definitions of Memory Mapped Register Addresses */
#define IMSK_MMR        0xff008504
#define IPND_MMR        0xff008500
#define ICON_MMR        0xff008510
#define IMAP1_MMR       0xff008524
#define IMAP2_MMR       0xff008528
#define PRCB_MMR        0xff008700


  /* Control table definition */
#define    IPB0                 0
#define    IPB1                 1
#define    DAB0                 2
#define    DAB1                 3
#define    IMAP0                4
#define    IMAP1                5
#define    IMAP2                6
#define    ICON                 7

  /* CX specific definitions */
#define    MCON0                8
#define    MCON1                9 
#define    MCON2                10
#define    MCON3                11
#define    MCON4                12
#define    MCON5                13
#define    MCON6                14
#define    MCON7                15
#define    MCON8                16
#define    MCON9                17
#define    MCON10               18
#define    MCON11               19
#define    MCON12               20
#define    MCON13               21
#define    MCON14               22
#define    MCON15               23

  /* JX specific definitions */
#define    PMCON0_1              8
#define    PMCON2_3             10
#define    PMCON4_5             12
#define    PMCON6_7             14
#define    PMCON8_9             16
#define    PMCON10_11           18
#define    PMCON12_13           20
#define    PMCON14_15           22

#define    BPCON                24
#define    TC                   25
#define    BCON                 26
#define    RESERVED             27

typedef struct {
     unsigned    control_reg[28];     
} CONTROL_TABLE; 


 /* Fault Table Constructs */

   /* Fault definition */
typedef struct {
   void         (*flt_hndlr)();     
   int          magic_number;
} FAULT_TABLE_ENTRY;

 

  /* fault record */
typedef struct {
   unsigned int  pc;         /* process controls           */
   unsigned int  ac;         /* arithmetic controls        */
   unsigned int  type;       /* type, subtype, and flags   */
   unsigned int  *addr;      /* address of faulting instr  */
   }FAULT_RECORD;

/* Fault Procedure Entries */
#define PARALLEL      0
#define TRACE         2
#define OPERATION     4
#define ARITHMETIC    6
#define CONSTRAINT    10
#define PROTECTION    14
#define TYPE          20
 

typedef struct {
     unsigned    fault_entry[21];     
} FAULT_TABLE; 
 

 


/* processor priority definition*/
#define PRIORITY0  0      
#define modify_pc_priority(pri) \
	   ((pri>-1 && pri<32)?i960_modpc(0x001F0000,((pri)<<16)):-1)                                    



/* DMA Controller definitions */
#define DMAC 2

    /* channel selection */
#define CH_0  0
#define CH_1  1
#define CH_2  2

    /* bit modification */
#define SET   1
#define CLR   0

/* Interrupt table definition */
#define    IPND     0
#define    IMSK     1

typedef struct {
     unsigned long    pending_priorities;
     unsigned long    pending_interrupts[8];  
     void             (*interrupt_proc[248])();  
} INTERRUPT_TABLE;


   /* ensures that the processors state is retained and replaced */
   /* after the interrupt handler has completed.                 */
#pragma isr increment_var_isr 


/* System Procedure Table definition */
 

/* SYSTEM PROCEDURE ENTRIES */
/* Definitions can all be given for each of the 260 procedure entries */
/* Entries 220-259 are reserved by the monitor */
#define ENTRY0     12 
#define ENTRY1     13
#define ENTRY2     14
#define ENTRY3     15
#define ENTRY4     16
#define ENTRY5     17

/* System Procedure Table definition */

typedef struct {
     unsigned int  sys_entry[260];
}SYS_PROC_TABLE;     


/* PRCB definition */
typedef struct {
     FAULT_TABLE        *fault_table_adr;
     CONTROL_TABLE      *cntrl_table_adr;
     unsigned           init_ac_reg;
     unsigned           fault_config;
     INTERRUPT_TABLE    *interrupt_table_adr;
     SYS_PROC_TABLE     *sys_proc_table_adr;
     unsigned           reserved;
     void               *interrupt_stack;
     unsigned           inst_cache_config;
     unsigned           reg_cache_config;
} PRCB;

/*  Gets prcb address; uses a system procedure call to #245 */
#pragma system (get_prcb_adr = 245)
PRCB *get_prcb_adr(void); 



    /* System control type definitions */
#define    REQ_INTERRUPT        0
#define    INVALIDATE_ICACHE    1
#define    CONFIGURE_ICACHE     2
#define    REINITIALIZE         3
#define    LOAD_CONTROL_REG     4

   /* System control field 1 values used with CONFIGURE_ICACHE */
#define    ENABLE_ICACHE        0
#define    DISABLE_ICACHE       1
#define    LOCK_ICACHE          4
#define    LOCK_HALF_ICACHE     6

   /* System control field 1 values used with LOAD_CONTROL_REG */
#define    BREAK_POINT_GROUP    0
#define    INTERRUPT_GROUP      1
#define    MCON_0_TO_3          2
#define    MCON_4_TO_7          3  
#define    MCON_8_TO_11         4
#define    MCON_12_T0_15        5
#define    CONTROL_GROUP        6     

   /* System control macro */
#define    system_control(type, f_1, f_2, f_3, f_4) \
	       i960_sysctl (((type)<<8)|(f_1)|((f_2)<<16), f_3, f_4)




 
/*  DCCTL CONTROL CONSTANTS  */
/*  The dcctl instruction has several functions:   */

#define    DISABLE_DCACHE       0
#define    ENABLE_DCACHE        1
#define    INVALIDATE_DCACHE    2
#define    COHERENT_DCACHE      3
#define    DCACHE_STATUS        4
#define    DCACHE_TO_MEMORY     6


/* DCACHE_CONTROL MACRO   */
#define    dcctl_control( f_1, f_2, f_3 ) \
               i960_dcctl ( f_1 , f_2, f_3 )
 




#endif _H_SYSTEM
