/*******************************************************************
 *                                                                 *
 * File: cyint.c                           Date: 3/11/95           *
 *                                                                 *
 * Disclaimer:                                                     *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY,   *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR       *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no     *
 *              guarantee or representations regarding the use     *
 *              of, or the results of the use of, the software     *
 *              and documentation in terms of correctness,         *
 *              accuracy, reliability, currentness, or             *
 *              otherwise,  and you rely on the software,          *
 *              documentation, and results solely at your own      *
 *              risk.                                              *
 *                                                                 * 
 * Overview:                                                       *
 *                                                                 *
 * The purpose of this program is to show the steps required in    *
 * dealing with an interrupt.                                      *
 *                                                                 *
 * Revision History:                                               *
 * ------------------------------------------------------------    *
 *******************************************************************/
#include <stdio.h>
#include "system.h" 
#include "t85c36.h" 


/* function declarations */
extern int  increment_var_isr(void);

  /* variable declarations */
unsigned volatile int interrupt_counter = 0;

/*******************************************************************
 * File:  cyint.c                          Author:  Intel Corp.    *
 *                                                                 *
 * Procedure:  Main                                                *
 *                                                                 *
 * Description:                                                    *
 *                                                                 *
 *     This code will perform the following steps in handling of   *
 * a harware interrupt.                                            *
 *        1.  modify the ICON register                             *
 *        2.  modify the IMAP register                             *
 *        3.  caching of vector and interrupt handling procedure   *
 *        4.  lower processor priority                             *
 *        5.  modify the IMSK register                             *
 *        6.  clear the IPND register                              *
 *        7.  generate the hardware interrupt                      *
 *                                                                 *
 *******************************************************************/
main()
{ 
                        /* Temporary variables                     */
    unsigned int      i,  

                        /* Pointer to interrupt service routine    */
                      *proc_addr;

                        /* Pointer to prcb address                 */
    PRCB              *ptr_to_prcb;

                        /* Pointer to interrupt table address      */
    INTERRUPT_TABLE   *ptr_to_int_tbl;

                        /* Pointer to control table  address       */
    CONTROL_TABLE     *ptr_to_cntrl_tbl;

                        /* Pointer to internal data ram memory     */
    unsigned int      *dram = (unsigned int *)0x0; 
     
                        /* pointer to ICON memory mapped address   */      
    unsigned long     *icon_addr  = (unsigned long *)ICON_MMR, 
                        /*  pointer to IMAP1 memory mapped address */
                      *imap1_addr = (unsigned long *)IMAP1_MMR;
                         

/*  1. Get the pointer to the PRCB address.  The JX has the PRCB at a     */
/*     memory mapped location and the CX requires a system call to 245.   */
/*     differnt code.                                                     */

       #ifdef __i960_CF__ 
 
          ptr_to_prcb = get_prcb_adr();
          
       #endif

     

/*  2. Get the pointer to the interrupt table and control table.          */
/*     Only needed for CX since JX has memory mapped registers.           */

       #ifdef __i960_CF__ 
 
          ptr_to_int_tbl = ptr_to_prcb->interrupt_table_adr;
          ptr_to_cntrl_tbl = ptr_to_prcb->cntrl_table_adr;
 
       #endif

/*  3. Set the ICON register:                                             */

       #ifdef __i960_CF__ 
          /* Dedicated interrupt, falling edge activated, global          */
          /* interrupts disabled, mov to r3 mask unchanged, fetch         */
          /* vector enable fast, fast sampling, DMA suspend on Interrupt. */       
 
        	 ptr_to_cntrl_tbl -> control_reg[ICON] = 0xe7fc;
 
       #endif

       #ifdef __i960_JF__
          /* Dedicated interrupt, falling edge activated, global          */
          /* interrupts disabled, mov to r3 mask unchanged, fetch         */
          /* vector enable fast, fast sampling.                           */
 
                 *icon_addr = 0x67fc;
 
       #endif

/*  4. Set the IMAP1 register:                                            */
/*     vector number E2 (226) for external interrupt xint6                */

       #ifdef __i960_CF__  
 
          ptr_to_cntrl_tbl -> control_reg[IMAP1] = 
                     (ptr_to_cntrl_tbl -> control_reg[IMAP1] & 0xf0ff) | 0x0e00;

       #endif

       #ifdef __i960_JF__

          *imap1_addr = (*imap1_addr & 0xf0ff) | 0x0e00;
 
       #endif


/*  5. Store the vector number in internal data ram memory                */ 
/*     Last 2 bits must end with 10 inorder to be cached properly.        */   
       
       dram[0xe] = (unsigned int)increment_var_isr | (unsigned int)0x2; 
   
/*  6. Load and lock the interrupt procedure in the instruction           */ 
/*     cache.                                                             */

       proc_addr = (unsigned int *)increment_var_isr;
       system_control(CONFIGURE_ICACHE, LOCK_HALF_ICACHE, 0,        
                                                (void *) proc_addr ,0);
        
        
/*  7. Store control registers with sysctl.                               */
/*     Only needed in the CX to reload registers                          */

       #ifdef __i960_CF__ 
 
          system_control(LOAD_CONTROL_REG, INTERRUPT_GROUP, 0, 0, 0);

       #endif


/*  8. Change processor priority to 0 so that our interrupt               */
/*     which is at priority 28 will be processed                          */

       modify_pc_priority(PRIORITY0);


/*  9. Set IMSK register to accept interrupts on external                 */
/*     interrupt pin 6                                                    */
  
       i960_mod_reg(IMSK, 0x40);

 
/* 10. Clear the interrupt pending register IPND                          */
    
       i960_mod_reg(IPND, 0); 
       
/* 11. Enable global interrupts                                           */

       interrupt_enable();

 
/* 12. Reload the icon register with the new value                        */
/*     This is only necessary for the CX processor.                       */
       
       #ifdef __i960_CF__ 

          system_control(LOAD_CONTROL_REG, INTERRUPT_GROUP, 0, 0, 0);
        
       #endif

/* 13. Initialize the 82c36:                                              */
/*     on the CYCLONE evaluation board all three timers are               */
/*     hardwired to xint6.  This example uses timer2                      */

       init_85c36_timer();

/* 14. Demonstrate that the interrupt counter value in the                */
/*     interrupt service routine is actually being incremented            */

       for(i=0;i<9;i++)
          {
            printf("interrupt count = %d\n",interrupt_counter);
            spinwait(10000);         /* specialized delay loop */
          } 


}   /* end main */      
 


 