/****************************************************************
 *                                                              *
 * File: t85c36.c                       Date: 3/11/95           *
 *                                                              *
 * Disclaimer:                                                  *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR    *
 *              FITNESS FOR A PARTICULAR PURPOSE, and makes no  *
 *              guarantee or representations regarding the use  *
 *              of, or the results of the use of, the software  *
 *              and documentation in terms of correctness,      *
 *              accuracy, reliability, currentness, or          *
 *              otherwise,  and you rely on the software,       *
 *              documentation, and results solely at your own   *
 *              risk.                                           *
 *                                                              *
 * Description:                                                 *
 *              The purpose of this file is to provide the      *
 *              functions necessary to issue a JX or CX         *
 *              Microprocessor instruction which controls       *
 *              the on-board timers.                            *
 *                                                              *
 * Revision History:                                            *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/

 #include "t85c36.h" 

    /* variable declarations */
static unsigned int interrupt_timer = 2;
static unsigned int interrupt_irq = 2;


/****************************************************************
 *                                                              *
 * File: t85c36.c                       Date: 3/11/95           *
 *                                                              *
 * Procedure:   init_85c36_timer                                *
 *                                                              *
 *                                                              *
 * Description:                                                 *  
 *              Initialize the 85c36 cio registers              *
 *              and initializes the cio count.                  *
 *                                                              * 
 *    Inputs  - None                                            *
 *    Outputs - None                                            *
 *                                                              *
 * Revision History:                                            *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/

void init_85c36_timer()
{
    unsigned char reg;
    int counter = 2;
    int init_time = FIVE_MS * CRYSTALTIME;

    
    /* Reset CIO state machine */
    reg = CIO->ctrl;    /* Put CIO in reset state or state 0 */
    CIO->ctrl = 0;      /* Put CIO state 0 or 1 */
    reg = CIO->ctrl;    /* Put CIO in state 0 */


    start_timer(counter, init_time);   /* begin timers */
     
    spinwait(1000000);      /* delay loop */

}  /* end init_85c36_timer */


/****************************************************************
 *                                                              *
 * File: t85c36.c                       Date: 3/11/95           *
 *                                                              *
 * Procedure:   start_timer                                     *
 *                                                              *
 * Description:                                                 *  
 *              initialize the 85C36 interrupt,                 *
 *              cio registers, and cio count.                   *
 *                                                              * 
 *    Inputs  - counter                                         *
 *              init_time                                       *   
 *    Outputs - None                                            *
 *                                                              *
 * Revision History:                                            *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/
 
void start_timer(int counter, int init_time)
{
    unsigned char reg;

    CIO->ctrl = 0x0a+counter;       /* 0x0a = Timer  Cmd & Status */
    CIO->ctrl = 0xe0;               /* Clear IE, set Gate Cmd Bit */

    CIO->ctrl = 0x1c+counter;       /* 0x1c = Timer  Mode Specification */
    CIO->ctrl = 0x80;               /* continuous, single-shot, no ext */

    CIO->ctrl = 0x16+(counter * 2);        /* 0x16 = Timer  TC, MS byte */
    CIO->ctrl = ((init_time >> 8) & 0xff); 
    CIO->ctrl = 0x17+(counter * 2);        /* 0x17 = Timer  TC, LS byte */
    CIO->ctrl = (init_time & 0xff); 

    CIO->ctrl = 0x0a+counter;            /* 0x0a = Timer  Cmd & Status */
    CIO->ctrl = 0x20;            /* Clear IP/IUS, set Gate Cmd Bit */

    if (counter == interrupt_timer)
	{
	CIO->ctrl = 0x01;            /* 0x00 = Master Config Control */
	reg = CIO->ctrl;
	CIO->ctrl = 0x01;            /* 0x00 = Master Config Control */
	CIO->ctrl = reg | (0x40 >> counter); /* Clear timer link control bits */

	CIO->ctrl = 0x0a+counter;    /* 0x0a = Timer  Cmd & Status */
	CIO->ctrl = 0xc6;            /* Set IE, set Gate Cmd Bit */

	CIO->ctrl = 0x00;            /* 0x00 = Master Int Control */
	CIO->ctrl = 0x80;      /* Set master int enable bit */
	}
} /* end start_timer */


/****************************************************************
 *                                                              *
 * File: t85c36.c                       Date: 3/11/95           *
 *                                                              *
 * Procedure:   spinwait                                        *
 *                                                              *
 *                                                              *
 * Description:                                                 *  
 *              Delay loop needed for execution of interrupt    *
 *              handling. Tricks the complier into not removing *
 *              this procedure when optimizing the code.        * 
 *                                                              *
 *    Inputs  - count                                           *
 *    Outputs - None                                            *
 *                                                              *
 * Revision History:                                            *
 *                                                              *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/
void spinwait(int count)
{
      /* Variable declaration */
  volatile unsigned foo;  

  for(;count--;)      /* Delay loop */
    foo = count;      /* assisgnment is necessary in order to fool the *
                       * compiler into not eliminating this procedure. */
} /* end spinwait */

