/****************************************************************
 *                                                              *
 * File: timers.h                        Date: 12/13/94         *
 *                                                              *
 * Disclaimer:                                                  *
 *              Intel provides this AS IS, WITHOUT ANY WARRANTY *
 *              INCLUDING THE WARRANTY OF MERCHANTABILITY OR    *
 *              FITNESS FOR A PARTICULAR PURPOSE,  and makes    *
 *              no guarantee or representations regarding the   *
 *              use of, or the results of the use of, the       *
 *              software and documentation in terms of          *
 *              correctness,  accuracy, reliability,            *
 *              currentness, or otherwise,  and you rely on the *
 *              software, documentation, and results solely     *
 *              at your own risk.                               *
 *                                                              *
 * Description:                                                 *
 *              This file contains constant and procedure       *
 *              definitions for the code located in timer.c.    *
 *              The user should note the declaration of the     *
 *              structure  TIMER_MMR_TYPE.  This structure      *
 *              contains all the necessary registers associated *
 *              with a timer and points directly to the MMR     *
 *              registers.  This is possible since all timer    *
 *              registers are contiguous in MMR memory space.   *
 *                                                              *
 * Revision History:                                            *
 * ------------------------------------------------------------ *
 *                                                              *
 ****************************************************************/
#include "system.h"

#ifndef _H_TIMERS
#define _H_TIMERS

/* address for timer0 */
#define TMR0_MMR_BASE   0xff000300 
/* address for timer1 */ 
#define TMR1_MMR_BASE   0xff000310  

/* Definitions for TMR register bits */

      /* Timer Enable/Disable (bit 1)  */
#define TMR_ENABLE      0x2
#define TMR_DISABLE     0x0
  
      /* Timer Reload (bit 2)  */
#define TMR_RLD_ENABLE  0x4
#define TMR_RLD_DISABLE 0x0

      /* Timer Write Control (bit 3)  */
#define TMR_SUPCTL      0x8
#define TMR_ALLCTL      0x0

      /* Timer Input Clock selects (bits 4 & 5)  */
#define TMR_TICK_PER_8BCLKS 0x30    
#define TMR_TICK_PER_4BCLKS 0x20
#define TMR_TICK_PER_2BCLKS 0x10
#define TMR_TICK_PER_1BCLKS 0x0

   /* Definitions for IMSK Registers */
#define TMR_MASK_INT    0x0
#define TMR_UNMASK_INT  0x1


/* Timer register definitions */
typedef struct {
long TRR;
long TCR;
long TMR;
} TIMER_MMR_TYPE;


/*  DECLARE PROTOTYPE OF TIMER FUNCTIONS */
extern void timer_enable_int(TIMER_MMR_TYPE *tptr, long enable); 
extern void timer_map_intvec(TIMER_MMR_TYPE *tptr, long priority,
                             void *rtn_addr);
extern void timer_configure(TIMER_MMR_TYPE *tptr, long ctlwd,
                             long count, long reldcnt);
extern void timer_enable(TIMER_MMR_TYPE *tptr, long enable);
extern void timer_start(TIMER_MMR_TYPE *tptr, long count);
extern int timer_expired(TIMER_MMR_TYPE *tptr);

#endif /* _H_TIMERS */
