/*(cb*/
/*******************************************************************************
 *
 * Copyright (c) 1993, 1994 Intel Corporation
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as "not part of the original" any modifications
 * made to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to
 * distribution of the software or the documentation without specific,
 * written prior permission.
 *
 * Intel Corporation provides this AS IS, WITHOUT ANY WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY
 * OR FITNESS FOR A PARTICULAR PURPOSE.  Intel makes no guarantee or
 * representations regarding the use of, or the results of the use of,
 * the software and documentation in terms of correctness, accuracy,
 * reliability, currentness, or otherwise; and you rely on the software,
 * documentation and results solely at your own risk.
 *
 * IN NO EVENT SHALL INTEL BE LIABLE FOR ANY LOSS OF USE, LOSS OF BUSINESS,
 * LOSS OF PROFITS, INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES
 * OF ANY KIND.  IN NO EVENT SHALL INTEL'S TOTAL LIABILITY EXCEED THE SUM
 * PAID TO INTEL FOR THE PRODUCT LICENSED HEREUNDER.
 *
 ******************************************************************************/
/*)ce*/

#include "common.h"
#include "i960.h"
#include "this_hw.h"
#include "c145_eep.h"

extern int send_sysctl(), strcpy(), strcat(), sprtf();

extern int boot_g0, dram_size;
extern char arch_name[], step_string[];
extern PRCB *prcb_ptr;

/*-------------------------------------------------------------
 * Function:      static void init_cio()
 *
 * Action:        Sets up CIO's bit ports, used for board
 *                control.
 *-------------------------------------------------------------*/
void
init_cio()
{
/***************************************************************
*
* DO NOT use a software reset command on the CIO.  The software
* reset command causes the clock speed selects (FREQ0, FREQ1, and
* FREQ2) to glitch causing unpredictable results in the Clock
* PLL device.
*
*/
	static unsigned char cmds[] = {
	    /*	reg#	value					*/
	    /*	-----	-----					*/
	    /*	Unreset CIO					*/
		0x00,   0x00,	/* Clear reset bit,
				   disable interrupts		*/

	    /*	Disable ports A, B, and C			*/
		0x01,	0x00,	/* Mast Conf Ctrl -disable ports*/

	    /*	Configure CIO port A - Processor Module options	*/
		0x20,	0x00,	/* Mode spec - bit port		*/
		0x21,	0x00,	/* Handshake spec - not used	*/
		0x22,	0x00,	/* Polarity - all bits non-inverting*/
		0x23,	0xff,	/* Direction - all bits inputs	*/
		0x24,	0x00,	/* Spec i/o ctrl - normal inputs*/
		0x25,	0x00,	/* Pattern matching registers	*/
		0x26,	0x00,	/* Pattern matching not used	*/
		0x27,	0x00,

	    /*	Configure CIO port B - Serial EEPROM's clk/data	*/
		0x28,	0x00,	/* Mode spec - bit port		*/
		0x29,	0x00,	/* Handshake spec - not used	*/
		0x2a,	0x00,	/* Polarity - no inverting	*/
		0x2b,	0xc0,	/* Dir - unused bits inputs	*/
		0x2c,	0x2a,	/* Data lines open drain	*/
		0x2d,	0x00,	/* Pattern matching registers	*/
		0x2e,	0x00,	/* Pattern matching not used	*/
		0x2f,	0x00,
		0x0e,	0xea,	/* Data reg - SCL's low, let	*/
				/* data lines float high.	*/

	    /*	Configure CIO port C - Misc control bits	*/
		0x06,	0x07,	/* Data dir - bit3 out, 0-2 in	*/
		0x0f,	0x00,	/* Data reg - pull RECONFIG low	*/

	    /*	Enable ports A, B, and C			*/
		0x01,	0x94	/* Mast Conf Ctrl - Enable ports*/
	};
	int i;
	volatile unsigned char junk;

	/* Reset CIO state machine */
	junk = CIO->ctrl;	/* Put CIO in reset state or state 0 */
	CIO->ctrl = 0;		/* Put CIO state 0 or 1 */
	junk = CIO->ctrl;	/* Put CIO in state 0 */

	for ( i = 0; i < sizeof(cmds); i++ ){
		CIO->ctrl = cmds[i];
	}
	return;
}

/*-----------------------------------------------------------------
 * Function:      static void init_squall()
 *
 * Action:        Checks to see if the squall module is installed
 *                and sets up MCON and interrupt info if so.
 *
 * Note:	  init_cio() must be called before this routine, so
 *		  that data port B may be used to access the eeprom!
 *-----------------------------------------------------------------*/
void
init_squall()
{
	SQUALL_EEPROM_DATA squall_data;

#if defined(__i960KA)||defined(__i960KB)||defined(__i960SA)||defined(__i960SB) \
 || defined(__i960KA__) || defined(__i960KB__) \
 || defined(__i960SA__) || defined(__i960SB__) \
 || (_80960KA == 1) || (_80960KB == 1) || (_80960SA == 1) || (_80960SB == 1)
	if (eeprom_read (SQUALL_EEPROM, GENERIC_DATA_OFFSET,
			 (void *) &squall_data, GENERIC_DATA_SIZE) != OK)
	 {
	 /* nothing to do for KX/SX chips at present */ /* self test ???*/
	 }
#endif

#if defined(__i960JA) || defined(__i960JA__) || (_80960JA == 1) || \
         defined(__i960JD) || defined(__i960JD__) || (_80960JD == 1) || \
         defined(__i960JF) || defined(__i960JF__) || (_80960JF == 1)

	unsigned long icon;

	/* read current value of the ICON */
	icon = *(unsigned long *)ICON_ADDR;

	/*
	 * Try reading configuration info from squall eeprom.
	 * If read fails, no squall module installed, just return
	 */
	if (eeprom_read (SQUALL_EEPROM, GENERIC_DATA_OFFSET,
			 (void *) &squall_data, GENERIC_DATA_SIZE) != OK)
	    /* don't do anything, just return */
	    return;
	else
	    {
	    /* MCON won't change always a 32 bit region */

	    /* Clear detection mode bits in ICON */
	    icon &= ~(EDGE_DETECT(SQ_IRQ0_XINT_PIN,1) |
		    EDGE_DETECT(SQ_IRQ1_XINT_PIN,1));

	    /* Set ICON bits according to value in eeprom */
	    icon |=
	      EDGE_DETECT(SQ_IRQ0_XINT_PIN,(squall_data.intr_det_mode & IRQ_0_MASK)
== IRQ_0_FALLING_EDGE);

	    icon |=
	      EDGE_DETECT(SQ_IRQ1_XINT_PIN,(squall_data.intr_det_mode & IRQ_1_MASK)
== IRQ_1_FALLING_EDGE);
	    }

	/* Store the new ICON */
	*(unsigned long *)ICON_ADDR = icon;

	return;
#endif

#if defined(__i960CA) || defined(__i960CA__) || (_80960CA == 1) 

	CONTROL_TABLE *control_tab = prcb_ptr->cntrl_table_adr;

	/*
	 * Try reading configuration info from squall eeprom.
	 * If read fails, no squall module installed, set MCON
	 * word to NO_SQUALL.
	 */
	if (eeprom_read (SQUALL_EEPROM, GENERIC_DATA_OFFSET,
			 (void *) &squall_data, GENERIC_DATA_SIZE) != OK)
	    control_tab->control_reg[MCON12] = NO_SQUALL;
	else
	    {
	    /* Set MCON word in control table */
	    control_tab->control_reg[MCON12] = squall_data.mcon_byte_0 |
					      (squall_data.mcon_byte_1 << 8) |
					      (squall_data.mcon_byte_2 << 16) |
					      (squall_data.mcon_byte_3 << 24);
	    /* Clear detection mode bits in ICON */
	    control_tab->control_reg[ICON] &=
	      ~(EDGE_DETECT(SQ_IRQ0_XINT_PIN,1) |
		EDGE_DETECT(SQ_IRQ1_XINT_PIN,1));

	    /* Set ICON bits according to value in eeprom */
	    control_tab->control_reg[ICON] |=
	      EDGE_DETECT(SQ_IRQ0_XINT_PIN,(squall_data.intr_det_mode & IRQ_0_MASK) == IRQ_0_FALLING_EDGE);
	    control_tab->control_reg[ICON] |=
	      EDGE_DETECT(SQ_IRQ1_XINT_PIN,(squall_data.intr_det_mode & IRQ_1_MASK) == IRQ_1_FALLING_EDGE);
	    }

	/* Re-load the modified sections of the control table into the CPU */
	send_sysctl(0x401, 0, 0);	/* Reload ICON */
	send_sysctl(0x405, 0, 0);	/* Reload MCON 12-15 */
	return;
#endif
}

/*-------------------------------------------------------------
 * Function:    void cpu_identify(void)
 *
 * Action:      Modify the arch_name string to include the actual 
 *              clock frequency the CPU is running at.
 *		Note that the macros which obtain the processor type
 *		and clock frequency rely on the cio being already
 *		initialized
 *-------------------------------------------------------------*/
void
cpu_identify(void)
{
    int speed;
	unsigned char buffer[20];

    speed = GET_CLK_FREQ();

    switch (speed)
    {
	case FREQ_4_MHZ:
	    strcat(arch_name, " at 4 MHz");
	    break;
	case FREQ_8_MHZ:
	    strcat(arch_name, " at 8 MHz");
	    break;
	case FREQ_16_MHZ:
	    strcat(arch_name, " at 16 MHz");
	    break;
	case FREQ_20_MHZ:
	    strcat(arch_name, " at 20 MHz");
	    break;
	case FREQ_25_MHZ:
	    strcat(arch_name, " at 25 MHz");
	    break;
	case FREQ_33_MHZ:
	    strcat(arch_name, " at 33 MHz");
	    break;
	case FREQ_40_MHZ:
	    strcat(arch_name, " at 40 MHz");
	    break;
	case FREQ_50_MHZ:
	    strcat(arch_name, " at 50 MHz");
	    break;
	default:
	    strcat(arch_name, " at ?? MHz");
	    break;
    }

	sprtf(buffer, 20," with%dMB DRAM", (char)(dram_size >> 20));
	strcat(arch_name, buffer);
    return;
}
