/*(cb*/
/*******************************************************************************
 *
 * Copyright (c) 1993, 1994 Intel Corporation
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as "not part of the original" any modifications
 * made to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to
 * distribution of the software or the documentation without specific,
 * written prior permission.
 *
 * Intel Corporation provides this AS IS, WITHOUT ANY WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY
 * OR FITNESS FOR A PARTICULAR PURPOSE.  Intel makes no guarantee or
 * representations regarding the use of, or the results of the use of,
 * the software and documentation in terms of correctness, accuracy,
 * reliability, currentness, or otherwise; and you rely on the software,
 * documentation and results solely at your own risk.
 *
 * IN NO EVENT SHALL INTEL BE LIABLE FOR ANY LOSS OF USE, LOSS OF BUSINESS,
 * LOSS OF PROFITS, INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES
 * OF ANY KIND.  IN NO EVENT SHALL INTEL'S TOTAL LIABILITY EXCEED THE SUM
 * PAID TO INTEL FOR THE PRODUCT LICENSED HEREUNDER.
 *
 ******************************************************************************/
/*)ce*/

#include "common.h"
#include "i960.h"
#include "cyjx.h"
#include "hdi_arch.h"

extern void set_mon_priority(unsigned int priority);
extern unsigned int get_mon_priority();
extern PRCB *get_prcbptr();
extern void enable_interrupt(), send_sysctl();
extern int serial_intr(), get_mask(), get_imap();
extern void fatal_error(), set_imap(), set_mask(), set_pending();
extern void set_lmar0(), set_lmmr0(), enable_dcache();
extern int get_lmar0(), get_lmmr0();

extern struct PRCB rom_prcb;

/************  C145/C146 CX BOARD-SPECIFIC CONSTANTS AND DATA  ***************/
/*********************************************************************
 * board identifier
 *********************************************************************/

/***************************************************************************
 ***************************************************************************
 *  For each module used, constants and data variables that pertain to that* 
 *  architecture must be modified with the  board specific definitions.    *
 ***************************************************************************
 ***************************************************************************/

int arch = ARCH_JX;            /* Change processor type to board */
                               /* capabilities                   */
char arch_name[32] = "JX";     /* add speed and mem size in cpu_identify() */
                               /* Change to board capabilities   */
char board_name[] = "Cyclone Baseboard";  /*  Change to board name */

const CONTROL_TABLE boot_control_table = {
	/* -- Group 0 -- Breakpoint Registers (reserved by monitor) */
	{0, 0, 0, 0,

	/* -- Group 1 -- Interrupt Map Registers */
	0, 0, 0,        /* Interrupt Map Regs (set by code as needed) */
	0x43bc,         /* ICON - dedicated mode, edge detected, enabled,
			 * mask unchanged, not cached, fast, DMA suspended */

	/* -- Groups 2-5 -- Bus Configuration Registers */
	REGION_0_CONFIG,
	0,
	REGION_2_CONFIG,
	0,
	REGION_4_CONFIG,
	0,
	REGION_6_CONFIG,
	0,
	REGION_8_CONFIG,
	0,
	REGION_A_CONFIG,
	0,
	REGION_C_CONFIG,
	0,
	REGION_E_CONFIG,
	0,

	/* -- Group 6 -- Breakpoint, Trace and Bus Control Registers */
	0,                      /* Reserved */
	0,                      /* BPCON Register (reserved by monitor) */
	0,                      /* Trace Controls  */
	REGION_TABLE_VALID(1)}  /* BCON Register  */
};

/* This is controlled by a jumper; thus it may be changed. */
/* If NMI is not used, the interrupt is connected to int 0. */
/* Breaks come in over both pin 0 ans NMI, entry.s takes care  of this */
#define BRKIV   0xf2            /* board interrupt vector is NMI */
#define BRK_PIN DUART_XINT_PIN  /* interrupt pin number */

extern void cpu_identify(void);
extern void init_cio();
extern void init_squall();
extern void init_eeprom();      /* sets up info about Flash Banks */


/*-------------------------------------------------------------
 * Function:      void init_hardware()
 *
 * Action:        Sets up target control hardware and peripherals.
 *-------------------------------------------------------------*/
/******************************************************************************
*******************************************************************************
* This procedure must intialize only the harware devices that are required    *
* by the montior.  Any other changes are done during the download to the      *
* target board.                                                               *
*******************************************************************************
*******************************************************************************/
/* default values contained in this procedure regarding the initialization of *
 * the board can be modified in this program.  Any special routines that are  *
 * required by the board for initialization are put in this procedure.        *
 ******************************************************************************/
void
init_hardware()
{
	extern void intr_entry();
 
	/* set data cache on for A memory region, DRAM */
    set_lmar0(0xa0000002);
    set_lmmr0(0xf0000001);
	enable_dcache();

	init_cio();             /* Initialize the Z8536 CIO */

	cpu_identify();         /* modify CPU string to include clock speed */
				/* must init CIO first! */

	init_eeprom();          /* establish operational parameters for
				   the various banks of Flash ROM */

  	init_squall();           /* Get Squall Module data */
    set_mask(0);            /* reset all interrupts */  
	set_pending(0);
    return;
}


/* Return the break interrupt vector number to the monitor.  */
/***************************************************************************
****************************************************************************
* This routine allows an interrupt to be generated when a break is         *
* received by the serial port.  This feature is not needed, but can use    *
* the UART to generate the interrupt.                                      *
****************************************************************************
***************************************************************************/
int
get_int_vector(int dummy)
{
    return(BRKIV);
}


/* Set up the interrupt map register to use the appropriate vector. */

/***************************************************************************
****************************************************************************
* This procedure needs no modification if BRK_PIN and BRKIV are defined    *
* correctly.                                                               *
****************************************************************************
***************************************************************************/

void
init_imap_reg(PRCB *prcb)
{
#if BRK_PIN < 4
    set_imap(0, (get_imap(0) & ~(0xf << BRK_PIN*4)) | ((BRKIV>>4) <<
BRK_PIN*4));
#else
    set_imap(1, (get_imap(1) & ~(0xf << (BRK_PIN-4)*4)) | ((BRKIV>>4)
<< (BRK_PIN-4)*4));
#endif

#define NEW_PRIORITY ((BRKIV >> 3) - 1)
    if (get_mon_priority() > NEW_PRIORITY)
	set_mon_priority(NEW_PRIORITY);

    enable_interrupt(BRK_PIN);
}

/*
 * Clear_break_condition is called when the monitor is entered due to a
 * interrupt from the serial port.  It should do any special processing
 * that has to happen in that situation.  For example, the interrupt
 * condition in the interrupt controller or the UART may have to be cleared.
 * Clear_break_condition returns TRUE if the interrupt was caused by
 * a break.  It returns FALSE if the interrupt was caused by some other
 * condition.  The monitor does not care about any conditions other than
 * break, but in some UARTs it is impossible to mask them without masking
 * the break interrupt.
 */
int
clear_break_condition(void)
{
    int flag;
    flag = serial_intr();
    return(flag);
}


/*-------------------------------------------------------------
 * Function:    void board_reset(void)
 *
 * Action:      Software board reset;
 *              Mask interrupts, reload the rom-based prcb, and
 *              reinitialize the CPU.
 *              This function does not return.
 *-------------------------------------------------------------*/
void
board_reset(void)
{
	extern void reinit();

	set_mask(0);         /* Reset all interrupts */
	set_pending(0);
	asm("ld _boot_g0, g11");
	send_sysctl(0x300, reinit, &rom_prcb);

	/* SHOULDN'T GET HERE */
	fatal_error(4,(int)"Board_reset",0,0,0);
}


void
board_go_user()
{
     register_set[REG_IMSK] |= 1 << BRK_PIN;
}

void
board_exit_user()
{
}
