/*(cb*/
/*******************************************************************************
 *
 * Copyright (c) 1993, 1994 Intel Corporation
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as "not part of the original" any modifications
 * made to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to
 * distribution of the software or the documentation without specific,
 * written prior permission.
 *
 * Intel Corporation provides this AS IS, WITHOUT ANY WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY
 * OR FITNESS FOR A PARTICULAR PURPOSE.  Intel makes no guarantee or
 * representations regarding the use of, or the results of the use of,
 * the software and documentation in terms of correctness, accuracy,
 * reliability, currentness, or otherwise; and you rely on the software,
 * documentation and results solely at your own risk.
 *
 * IN NO EVENT SHALL INTEL BE LIABLE FOR ANY LOSS OF USE, LOSS OF BUSINESS,
 * LOSS OF PROFITS, INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES
 * OF ANY KIND.  IN NO EVENT SHALL INTEL'S TOTAL LIABILITY EXCEED THE SUM
 * PAID TO INTEL FOR THE PRODUCT LICENSED HEREUNDER.
 *
 ******************************************************************************/
/*)ce*/

#include <string.h>

#include "common.h"
#include "i960.h"
#include "retarget.h"
#include "dbg_mon.h"
#include "hdi_errs.h"
#include "hdi_com.h"

#if defined(__i960_BIG_ENDIAN__)
   extern swap_data_byte_order(int, void *, int);
#endif /* __i960_BIG_ENDIAN__ */

extern int store_mem(ADDR, int mem_size, const void *data, int data_size, int verify);
extern int fill_mem(ADDR, int mem_size, const void *data, int data_size, int count);
extern leds(int,int), led_debug(char,int,int,int,int);
extern const unsigned short CRCtab[];

#define PUTBUF(msg) com_put_msg((const unsigned char *)&(msg), sizeof(msg))

/*****************************************************************************/
/*                                                                           */
/* FUNCTION NAME: parallel_download                                         */
/*                                                                           */
/*****************************************************************************/

static void
parallel_error(int led_val, int error)
{
    CMD_TMPL response;

        /*for (mem_size=0; mem_size<100; mem_size++) parallel_read(&data[mem_size], 1, NULL);
        com_init_msg();
        com_put_byte(PARALLEL_DOWNLOAD);
        com_put_byte(0xff);
        com_put_data(data, size);
        com_put_msg(NULL, 0);
		return; */   /* debug for new parallel target */
        /*com_init_msg();
        com_put_byte(PARALLEL_DOWNLOAD);
        com_put_byte(0xff);
        com_put_long(val);
        com_put_msg(NULL, 0); */

        blink(led_val);
		parallel_err();
        response.cmd = PARALLEL_DOWNLOAD;
		if (error != 0)
            response.stat = error;
		else
            response.stat = E_PARALLEL_BAD_FORMAT;
        PUTBUF(response);
}


void
parallel_download(const void *p_cmd)
{
#define PARALLEL_DELIMITER 0xff
#define DATA_BUF_SIZE 400
    unsigned char data[DATA_BUF_SIZE];

    ADDR addr;
    unsigned char byte_1, byte_2, format_byte, *rp = (unsigned char *)p_cmd + 2;
    unsigned int addr_value, mem_size, pattern_size, eeprom_mem, write_or_fill_cmd, data_size;
    unsigned short our_crc, msg_crc;
    CMD_TMPL response;

    /* init parallel port */
    parallel_init();

    format_byte = get_byte(rp);

    response.cmd = PARALLEL_DOWNLOAD;
    response.stat = OK;
    PUTBUF(response);

    /* just to see if we support parallel */
    if (format_byte == 0)
      return;

  leds(8,8);
  do 
  {  /* read parallel until 5 message */

	format_byte = our_crc = mem_size = pattern_size = 0;

    if (parallel_read(&format_byte, 1, NULL) == ERR)
		{
		parallel_error(9, E_PARALLEL_TIMEOUT);
        return;
		}

	if (format_byte == PARALLEL_DOWNLOAD)
		{
		/* end of parallel */
        PUTBUF(response);
		leds(0,8);
        return;
		}

    if ((format_byte != WRITE_MEM) && (format_byte != FILL_MEM))
        {
		parallel_error(7, E_PARALLEL_BAD_FORMAT);
        return;
        }

    write_or_fill_cmd = format_byte;

    if ((parallel_read((unsigned char *)&addr, 4, &our_crc) == ERR) ||
        (parallel_read((unsigned char *)&data_size, 4, &our_crc) == ERR) ||
		(parallel_read((unsigned char *)&byte_1, 1, &our_crc) == ERR) ||
        (parallel_read((unsigned char *)&byte_2, 1, &our_crc) == ERR) ||
        (parallel_read(&format_byte, 1, NULL) == ERR))
		{
		parallel_error(9, E_PARALLEL_TIMEOUT);
        return;
        }

    mem_size = (int)byte_1;
    pattern_size = (int)byte_2;
    if (format_byte != PARALLEL_DELIMITER)
		{
		parallel_error(7, E_PARALLEL_BAD_FORMAT);
        return;
        }

#if defined(__i960_BIG_ENDIAN__)
    swap_data_byte_order(4, &addr, 4);
    swap_data_byte_order(4, &data_size, 4);
#endif /* __i960_BIG_ENDIAN__ */
		
    
    /* now read the data  */
    if (write_or_fill_cmd == FILL_MEM)
		{
        if (parallel_read(data, pattern_size, &our_crc) == ERR)
            {
		    parallel_error(6, E_PARALLEL_TIMEOUT);
            return;
            }

#if defined(__i960_BIG_ENDIAN__)
        swap_data_byte_order(mem_size, data, pattern_size);
#endif /* __i960_BIG_ENDIAN__ */
		}
	else if (((eeprom_mem = is_eeprom(addr, (unsigned long)data_size)) == TRUE) || (mem_size > 1))
		{ /* eeprom or memory blocks */
		unsigned int i, increment = DATA_BUF_SIZE;

	    if ((eeprom_mem == TRUE) && (check_eeprom(addr, (unsigned long)data_size) != OK))
            {
		    parallel_error(5, E_EEPROM_PROG);
            return;
            }

        for (i = 0; i < data_size; i += DATA_BUF_SIZE)
            {

            /* last partial read */
            if (i + increment > data_size)
                increment = data_size - i;

            if (parallel_read(data, increment, &our_crc) == ERR)
                {
		        parallel_error(5, E_PARALLEL_TIMEOUT);
                return;
                }
#if defined(__i960_BIG_ENDIAN__)
            /* store_mem() expects target byte order, but we got it from
             * the host in little-endian byte order, so swap.
             */
            swap_data_byte_order(mem_size, data, increment);
#endif /* __i960_BIG_ENDIAN__ */
		    /* we verify memory with CRC check */
            if (store_mem((ADDR)(addr + (ADDR)i), mem_size, data, increment, FALSE) != OK)
                {
		        parallel_error(5, cmd_stat);
                return;
                }
            }
		}
	else if (eeprom_mem == FALSE)
		{    /* normal memory */
		/* test if writable memory */
		*(unsigned int *)addr = 0;
		addr_value = *(unsigned int *)addr;
		*(unsigned int *)addr = 0xa5a5a5a5;
		if (*(unsigned int *)addr != 0xa5a5a5a5 || addr_value != 0)
            {
		    parallel_error(4, E_WRITE_ERR);
            return;
            }
        if (parallel_read((unsigned char *)addr, data_size, &our_crc) == ERR)
            {
		    parallel_error(4, E_PARALLEL_TIMEOUT);
            return;
            }
#if defined(__i960_BIG_ENDIAN__)
        swap_data_byte_order(mem_size, (unsigned char *)addr, data_size);
#endif /* __i960_BIG_ENDIAN__ */
        }
    else 
		{ /* Partial eeprom - not allowd */
		parallel_error(3, E_EEPROM_ADDR);
        return;
		}

    if ((parallel_read((unsigned char *)&msg_crc, 2, NULL) == ERR) ||
        (parallel_read(&format_byte, 1, NULL) == ERR))
        {
		led_debug(0xa,(int)msg_crc, (int)format_byte, (int)our_crc,(int)data_size);
		parallel_error(2, E_PARALLEL_TIMEOUT);
        return;
        }
#if defined(__i960_BIG_ENDIAN__)
    swap_data_byte_order(2, &msg_crc, 2);
#endif /* __i960_BIG_ENDIAN__ */
    if (our_crc != msg_crc)
        {
		led_debug(0xb,(int)msg_crc, (int)format_byte, (int)our_crc,(int)data_size);
		parallel_error(2, E_PARALLEL_BAD_DATA_CHECKSUM);
        return;
        }
    if (format_byte != PARALLEL_DELIMITER)
        {
		led_debug(0xc,(int)msg_crc, (int)format_byte, (int)our_crc,(int)data_size);
		parallel_error(2, E_PARALLEL_BAD_FORMAT);
        return;
        }

    if (write_or_fill_cmd == FILL_MEM)
        {
        if (fill_mem(addr, mem_size, data, pattern_size, data_size) != OK)
            {
		    parallel_error(1, cmd_stat);
            return;
            }
        }

  } while (TRUE);
}

