/**************************************************
 *                                                *
 *  Header:  arch960.h                            *
 *                                                *
 *  Revision History:                             *
 *                                                *
 *     TJ  12/31/94  Creation                     *
 *                                                *
 *************************************************/

#ifndef _ARCH960_H_
#define _ARCH960_H_

#include "types.h"

/* definitions for PC register when creating new tasks */
/* simply the reset state of the PC register           */

#if defined(__i960CA__) || defined(__i960CF__)
#define INIT_PC 0xC01A2003

#elif defined(__i960JA__) || \
      defined(__i960JD__) || \
      defined(__i960JF__) || \
      defined(__i960HA__) || \
      defined(__i960HD__) || \
      defined(__i960HT__)


#define INIT_PC 0x001A2003

#else
#error Unsupported Architecture
#endif

/**************************************************
 *                                                *
 * General Purpose Register Constructs            *
 *                                                *
 *************************************************/

/* local register */
typedef struct tagLOCALREGS
{
  WORD r0;
  WORD r1;
  WORD r2;
  WORD r3;
  WORD r4;
  WORD r5;
  WORD r6;
  WORD r7;
  WORD r8;
  WORD r9;
  WORD r10;
  WORD r11;
  WORD r12;
  WORD r13;
  WORD r14;
  WORD r15;
} LOCALREGS;

/* global registers */
typedef struct tagGLOBREGS
{
  WORD g0;
  WORD g1;
  WORD g2;
  WORD g3;
  WORD g4;
  WORD g5;
  WORD g6;
  WORD g7;
  WORD g8;
  WORD g9;
  WORD g10;
  WORD g11;
  WORD g12;
  WORD g13;
  WORD g14;
  WORD g15;
} GLOBREGS;

/**************************************************
 *                                                *
 * Control Register Constructs                    *
 *                                                *
 *************************************************/

/* breakpoint control register prototype */
typedef union tagBKPTCON
{
  WORD Value;
  struct
  {
    WORD Reserved1 : 16;
    WORD DAB0En    : 2;
    WORD DAB0Mode  : 2;
    WORD DAB1En    : 2;
    WORD DAB1Mode  : 2;
    WORD Reserved2 : 8;
  } Bits;
} BKPTCON;

/* trace controls register prototype */
typedef union tagTRCCTL
{
  WORD Value;
  struct
  {
    WORD Reserved1 : 1;
    WORD InstMd    : 1;
    WORD BrMd      : 1;
    WORD CallMd    : 1;
    WORD RetMd     : 1;
    WORD PreRetMd  : 1;
    WORD SupMd     : 1;
    WORD MarkMd    : 1;
    WORD Reserved2 : 9;
    WORD InstFl    : 1;
    WORD BrFl      : 1;
    WORD CallFl    : 1;
    WORD RetFl     : 1;
    WORD PreRetFl  : 1;
    WORD SupFl     : 1;
    WORD MarkFl    : 1;
    WORD IPB0Fl    : 1;
    WORD IPB1Fl    : 1;
    WORD DAB0Fl    : 1;
    WORD DAB1Fl    : 1;
    WORD Reserved3 : 4;
  } Bits;
} TRCCTL;

/* bus control register prototype */
typedef union tagBCON
{
  WORD Value;
  struct
  {
    WORD TblValid  : 1;
    WORD InRAMProt : 1;
    WORD Reserved  : 30;
  } Bits;
} BUSCON;

/* arithmetic controls register prototype */
typedef union tagARITHCTL
{
  WORD Value;
  struct
  {
    WORD CondCd     : 3;
    WORD Reserved1  : 5;
    WORD Overflow   : 1;
    WORD Reserved2  : 3;
    WORD OverflwMsk : 1;
    WORD Reserved3  : 2;
    WORD NIF        : 1;
    WORD Reserved4  : 16;
  } Bits;
} ARITHCTL;

/* process controls register prototype */
typedef union tagPROCCTL
{
  WORD Value;
  struct
  {
    WORD TrcEn     : 1;
    WORD XMod      : 1;
    WORD Reserved1 : 8;
    WORD TrcFltPnd : 1;
    WORD Reserved2 : 2;
    WORD State     : 1;
    WORD Reserved3 : 2;
    WORD Priority  : 5;
    WORD Reserved4 : 11;
  } Bits;
} PROCCTL;

/* fault configuration register prototype */
typedef union tagFAULTCONFIG
{
  WORD Value;
  struct
  {
    WORD Manditory       : 1;
    WORD Reserved1       : 29;
    WORD UnalignedBusFlt : 1;
    WORD Reserved2       : 1;
  } Bits;
} FAULTCONFIG;

/* instruction cache configuration word */
typedef union tagICACHECONFIG
{
  WORD Value;
  struct
  {
    WORD Reserved1 : 16;
    WORD Disable   : 1;
    WORD Reserved2 : 15;
  } Bits;
} ICACHECONFIG;

/* register cache configuration word */
#if defined(__i960CA__) || defined(__i960CF__)

typedef union tagREGCACHECONFIG
{
  WORD Value;
  struct
  {
    WORD CacheLevel : 4;
    WORD Reserved   : 28;
  } Bits;
} REGCACHECONFIG;

#elif defined(__i960JA__) || \
      defined(__i960JD__) || \
      defined(__i960JF__)

typedef union tagREGCACHECONFIG
{
  WORD Value;
  struct
  {
    WORD Reserved1  : 8;
    WORD HighPrior  : 3;
    WORD Reserved2  : 21;
  } Bits;
} REGCACHECONFIG;

#elif defined(__i960HA__) || \
      defined(__i960HD__) || \
      defined(__i960HT__)

typedef union tagREGCACHECONFIG
{
  WORD Value;
  struct
  {
    WORD CacheLevel : 4;
    WORD Reserved1  : 4;
    WORD HighPrior  : 4;
    WORD Reserved2  : 20;
  } Bits;
} REGCACHECONFIG;

#else
#error Unsupported Architecture
#endif

/**************************************************
 *                                                *
 * Fault Table Constructs                         *
 *                                                *
 *************************************************/

/* fault table entry structure */
typedef struct tagFLTTBLENTRY
{
  union
  {
    WORD Procedure;
    struct
    {
      WORD Control :  2;
      WORD Proc    : 30;
    } Bits;
  } FltHandler;
  WORD Reserved;
} FLTTBLENTRY;

/* fault table definition */
typedef struct tagFLTTABLE
{
  FLTTBLENTRY ParallelFlt;
  FLTTBLENTRY TraceFlt;
  FLTTBLENTRY OperationFlt;
  FLTTBLENTRY ArithmeticFlt;
  FLTTBLENTRY FloatingPtFlt;
  FLTTBLENTRY ConstraintFlt;
  FLTTBLENTRY VirtualMemFlt;
  FLTTBLENTRY ProtectionFlt;
  FLTTBLENTRY MachineFlt;
  FLTTBLENTRY StructuralFlt;
  FLTTBLENTRY TypeFlt;
  FLTTBLENTRY Reserved1;
  FLTTBLENTRY ProcessFlt;
  FLTTBLENTRY DescriptorFlt;
  FLTTBLENTRY EventFlt;
  FLTTBLENTRY Reserved2[17];
} FLTTABLE;


/**************************************************
 *                                                *
 * Control Table Constructs                       *
 *                                                *
 *************************************************/

/* instrcution breakpoint prototypes */
typedef struct tagIPBX
{
  union
  {
    WORD IPAddr;
    struct
    {
      WORD Mode   : 2;
      WORD IPBits : 30;
    } Bits;
  } IP;
} IPBX;

/* data address breakpoint prototypes */
typedef WORD DABX;

/* interrupt map register prototypes */
typedef union tagIMAP
{
  WORD Value;
  struct
  {
    WORD X0       : 4;
    WORD X1       : 4;
    WORD X2       : 4;
    WORD X3       : 4;
    WORD Reserved : 16;
  } Bits;
} IMAP;

/* interrupt control register prototypes */
typedef union tagINTCON
{
  WORD Value;
  struct
  {
    WORD IMode        : 2;
    WORD DMode        : 8;
    WORD GlbIntEn     : 1;
    WORD MskOp        : 2;
    WORD VectCacheEn  : 1;
    WORD SmplMode     : 1;
    #if defined(__i960CA__) || defined(__i960CF__)
    WORD DMASusp      : 1;
    WORD Rerserved    : 16;

    #elif defined(__i960JA__) || \
          defined(__i960JD__) || \
          defined(__i960JF__) || \
          defined(__i960HA__) || \
          defined(__i960HD__) || \
          defined(__i960HT__)
    WORD Reserved     : 17;

    #else
    #error Unsupported Architecture
    #endif
  } Bits;
} INTCON;

/* region control entry prototypes for Cx and Jx */
#if defined(__i960CA__) || defined(__i960CF__)
typedef union tagMCON
{
  WORD Value;
  struct
  {
    WORD BurstEn    : 1;
    WORD RdyBtermEn : 1;
    WORD RdPipeEn   : 1;
    WORD NRAD       : 5;
    WORD NRDD       : 2;
    WORD NXDA       : 2;
    WORD NWAD       : 5;
    WORD NWDD       : 2;
    WORD BusWidth   : 2;
    WORD Reserved1  : 1;
    WORD ByteOrder  : 1;
    WORD DCacheEn   : 1;
    WORD Reserved2  : 8;
  } Bits;
} MCON;

#define CON MCON

#elif defined(__i960JA__) || \
      defined(__i960JD__) || \
      defined(__i960JF__) || \
      defined(__i960HA__) || \
      defined(__i960HD__) || \
      defined(__i960HT__)
typedef union tagPMCON
{
  WORD Value;
  struct
  {
    WORD Reserved1  : 22;
    WORD BusWidth   : 2;
    WORD Reserved2  : 8;
  } Bits;
} PMCON;

#define CON PMCON

#else
#error Unsupported Architecture
#endif

/* control table               */

/* NOTE:  The Jx does not use: */
/*          MCON1,  MCON3,     */
/*          MCON5,  MCON7,     */
/*          MCON9,  MCON11,    */
/*          MCON13, MCON15     */

typedef struct tagCONTROLTABLE
{
#if defined(__i960CA__) || defined(__i960CF__)
  IPBX    IPB0;
  IPBX    IPB1;
  DABX    DAB0;
  DABX    DAB1;

#elif defined(__i960JA__) || \
      defined(__i960JD__) || \
      defined(__i960JF__) || \
      defined(__i960HA__) || \
      defined(__i960HD__) || \
      defined(__i960HT__)
  WORD    Reserved1;
  WORD    Reserved2;
  WORD    Reserved3;
  WORD    Reserved4;

#else
#error Unsupported Architecture
#endif

  IMAP    IMAP0;
  IMAP    IMAP1;
  IMAP    IMAP2;
  INTCON  ICON;
  CON     MCON0;
  CON     MCON1;
  CON     MCON2;
  CON     MCON3;
  CON     MCON4;
  CON     MCON5;
  CON     MCON6;
  CON     MCON7;
  CON     MCON8;
  CON     MCON9;
  CON     MCON10;
  CON     MCON11;
  CON     MCON12;
  CON     MCON13;
  CON     MCON14;
  CON     MCON15;
  WORD    Reserved;

#if defined(__i960CA__) || defined(__i960CF__)
  BKPTCON BPCON;

#elif defined(__i960JA__) || \
      defined(__i960JD__) || \
      defined(__i960JF__) || \
      defined(__i960HA__) || \
      defined(__i960HD__) || \
      defined(__i960HT__)
  WORD    Reserved5;

#else
#error Unsupported Architecture
#endif

  TRCCTL  TC;
  BUSCON  BCON;
} CONTROLTABLE;

/**************************************************
 *                                                *
 * Interrupt Table Constructs                     *
 *                                                *
 *************************************************/

/* prototypes for interrupt table entries */
typedef union tagINTTBLENTRY
{
  void (*ISR) (void);
  struct
  {
    WORD Type   : 2;
    WORD Vector : 30;
  } Bits;
} INTTBLENTRY;

/* interrupt table */
typedef struct tagINTTBL
{
  WORD PendPriorities;
  WORD PendInts[8];
  INTTBLENTRY IntVecs0[236];
  WORD Reserved1[4];
  INTTBLENTRY NMI;
  WORD Reserved2[3];
  INTTBLENTRY IntVecs1[4];
} INTTBL;

/**************************************************
 *                                                *
 * System Procedure Table Constructs              *
 *                                                *
 *************************************************/

/* system procedure table entry prototypes */
typedef union tagSYSPROCTBLENTRY
{
  WORD Value;
  struct
  {
    WORD Type     : 2;
    WORD ProcAddr : 30;
  } Bits;
} SYSPROCTBLENTRY;

/* supervisor stack pointer prototype */
typedef union tagSUPSTCKPTR
{
  WORD Value;
  struct
  {
    WORD TrcCtl    : 1;
    WORD Reserved  : 1;
    WORD StackBase : 30;
  } Bits;
} SUPSTCKPTR;

/* system procedure table */
typedef struct tagSYSPROCTBL
{
  WORD Reserved1[3];
  SUPSTCKPTR SSP;
  WORD Reserved2[8];
  SYSPROCTBLENTRY ProcEnt[260];
} SYSPROCTBL;

/**************************************************
 *                                                *
 * Processor Control Block (PRCB) Constructs      *
 *                                                *
 *************************************************/

typedef struct tagPRCB
{
  FLTTABLE       *FltTblBase;
  CONTROLTABLE   *CtlTblBase;
  ARITHCTL       AC;
  FAULTCONFIG    FC;
  INTTBL         *IntTblBase;
  SYSPROCTBL     *SysProcTblBase;
  WORD           Reserved;
  WORD           ISP;
  ICACHECONFIG   ICCW;
  REGCACHECONFIG RCCW;
} PRCB;



#endif

