/*
 *   MCYCX.LD              August, 1992
 *
 *   This is the linker directive file to link an application to run
 *   under mon960 on the CYCX board.
 *   Programs linked with this file are downloaded to DRAM after the
 *   data area used by mon960.
 *   malloc() uses the symbols _heap_base and _heap_end to manage the heap.
 *   _heap_base is located at the end of program memory (.text + .data +.bss).
 *   _heap_size is defined as 0x2000.
 *   The C runtime stack starts at _heap_end and grows up.
 *   There must be enough room after the program memory in the dram for
 *   the program's heap and stack.
 *
 *   THIS IS NOT THE SAME MCYCX.LD THAT COMES WITH GNU/960!!!
 */

MEMORY
   {
      dram  :  org = 0xA0008000, len = 0x1f8000   /* 2M less monitor */
      isram :  org = 0x00000100, len = 0x300
   }


SECTIONS
   {
      .text :
         {
         } >dram

      .data :
         {
         } >dram

      .bss : 
         {
         } >dram
   }

/* Arithmetic controls location when using floating point library. */
fpem_CA_AC = 0x100;

/* kernel variables added for TOMOS */
_abTaskPrior      = 0x110;   /* bit set for each ready task */
_abTaskDelayed    = 0x130;   /* bit set for each delayed task */
_wHighestPriorRdy = 0x150;   /* integer value of highest priority ready task */

/* Bounds of heap: */
/* The heap may be placed in a separate memory region, if desired. */
_heap_size = 0x100000;   /* 1 Megabyte */
_heap_base = (_end + 0xf) & ~0xf;
_heap_end = _heap_base + _heap_size - 1;

/* _stackbase marks base of user stack */
/* stack is allocated following the heap. */
/* The stack may be based in a separate memory region, if desired. */
_stackbase = (_heap_end + 0x10) & ~0x0f;

STARTUP (crt960g.o)
HLL ()
/* SYSLIB (libmong.a) */
SYSLIB (libllg.a)
FLOAT

