/**************************************************
 *                                                *
 *  Module Name:  tasks.c                         *
 *                                                *
 *  Functions in this Module:                     *
 *    MyOtherTask()                               *
 *    MyTask()                                    *
 *                                                *
 *************************************************/

#include <stdlib.h>
#include "globals.h"
#include "inline.h"
#include "tasks.h"

#define DELAY 10000

/**************************************************
 *                                                *
 * Function:  MyOtherTask()                       *
 *                                                *
 * Description:                                   *
 *   Created by MyTask(); used to verify tasks    *
 *   can be dynamically created correctly.        *
 *                                                *
 * Prototype:  void MyOtherTask(void *pv)         *
 *                                                *
 * Parameters:                                    *
 *   pv        Pointer to user defined data       *
 *             structure for this task.           *
 *                                                *
 * Returns:                                       *
 *   Nothing.                                     *
 *                                                *
 * Revision History:                              *
 *                                                *
 *   TJ  12/30/94  Creation                       *
 *                                                *
 *************************************************/
void MyOtherTask(void *pv)
{
  int ii;

  /* let the world know we're here */
  printf("\n(MyOtherTask) : %03d\n", ptcbCur->wPriority);

  /* indicate MyOtherTask() will delay itself */
  printf("(MyOtherTask) : Delaying MyOtherTask()\n", 0);

  /* show the tick count before the delay */
  printf("\n(MyOtherTask) : wTicker %d\n", wTicker);
  TaskDelay(3000);
  for (ii = 0; ii < DELAY; ii++); /* make sure tick happens before */
                                  /* call to printf()              */

  /* display tick count after delay */
  printf("\n(MyOtherTask) : wTicker %d\n", wTicker);

  /* MyOtherTask quitting - tell 'em */
  printf("(MyOtherTask) : Done delaying MyOtherTask()\n", 0);
  
  /* MyOtherTask deleting itself */
  TaskDelete(TASK_PRIOR + 1);

  for (;;);
}

/**************************************************
 *                                                *
 * Function:  MyTask()                            *
 *                                                *
 * Description:                                   *
 *   Created at initialization.  This is the      *
 *   first task executed after multitasking       *
 *   begins.                                      *
 *                                                *
 * Prototype:  void MyTask(void *pv)              *
 *                                                *
 * Parameters:                                    *
 *   pv        Pointer to user defined data       *
 *             structure for this task.           *
 *                                                *
 * Returns:                                       *
 *   Nothing.                                     *
 *                                                *
 * Revision History:                              *
 *                                                *
 *   TJ  12/30/94  Creation                       *
 *                                                *
 *************************************************/
void MyTask(void *pv)
{
  int ii;

  /* tell the world we're here */
  printf("\n(MyTask) : %03d\n", ptcbCur->wPriority);

  /* dymanically create a new task - tell the world */
  printf("(MyTask) : Creating Task Priority %d\n", TASK_PRIOR + 1);

  TaskCreate(MyOtherTask, 0, TASK_STK_SZ, TASK_PRIOR + 1, TRUE);
  for (ii = 0; ii < DELAY; ii++);   /* make sure tick happens before */
                                    /* call to printf()              */

  /* MyTask() all done - tell the world */
  printf("\n(MyTask) : Done with MyTask(), wTicker %d\n", wTicker);

  /* MyTask() deleting itself - so sad */
  TaskDelete(TASK_PRIOR);
  for (ii = 0; ii < DELAY; ii++);

  /* we should never get here !!! */
  printf("(MyTask) : after task delete???, ptcbCur = %p\n", (WORD) ptcbCur);

  exit(2);
}



